import _typeof from "@babel/runtime-corejs3/helpers/typeof";
import _forEachInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/for-each";
import _Array$isArray from "@babel/runtime-corejs3/core-js-stable/array/is-array";
import _concatInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/concat";
import _lastIndexOfInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/last-index-of";
import _filterInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/filter";
import _indexOfInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/index-of";
import React from 'react';
export function toTitle(title) {
  if (typeof title === 'string') {
    return title;
  }
  return null;
}
export function getValuePropValue(child) {
  if (!child) {
    return null;
  }
  var props = child.props;
  if ('value' in props) {
    return props.value;
  }
  if (child.key) {
    return child.key;
  }
  if (child.type && child.type.isSelectOptGroup && props.label) {
    return props.label;
  }
  throw new Error("Need at least a key or a value or a label (only for OptGroup) for ".concat(child));
}
export function getPropValue(child, prop) {
  if (prop === 'value') {
    return getValuePropValue(child);
  }
  return child.props[prop];
}
export function isMultiple(props) {
  return props.multiple;
}
export function isCombobox(props) {
  return props.combobox;
}
export function isMultipleOrTags(props) {
  return props.multiple || props.tags;
}
export function isMultipleOrTagsOrCombobox(props) {
  return isMultipleOrTags(props) || isCombobox(props);
}
export function isSingleMode(props) {
  return !isMultipleOrTagsOrCombobox(props);
}
export function childrenToArray(children) {
  var _context;
  var ret = [];
  _forEachInstanceProperty(_context = React.Children).call(_context, children, function (c) {
    ret.push(c);
  });
  return ret;
}
export function toArray(value) {
  var ret = value;
  if (value === undefined) {
    ret = [];
  } else if (!_Array$isArray(value)) {
    ret = [value];
  }
  return ret;
}
export function getMapKey(value) {
  var _context2;
  return _concatInstanceProperty(_context2 = "".concat(_typeof(value), "-")).call(_context2, value);
}
export function preventDefaultEvent(e) {
  e.preventDefault();
}
export function findIndexInValueBySingleValue(value, singleValue) {
  var index = -1;
  for (var i = 0; i < value.length; i++) {
    if (value[i] === singleValue) {
      index = i;
      break;
    }
  }
  return index;
}
export function getLabelFromPropsValue(value, key) {
  var label;
  value = toArray(value);
  for (var i = 0; i < value.length; i++) {
    if (value[i].key === key) {
      label = value[i].label;
      break;
    }
  }
  return label;
}
export function getSelectKeys(menuItems, value) {
  var _context3;
  if (value === null || value === undefined) {
    return [];
  }
  var selectedKeys = [];
  _forEachInstanceProperty(_context3 = React.Children).call(_context3, menuItems, function (item) {
    if (item.type.isMenuItemGroup) {
      selectedKeys = _concatInstanceProperty(selectedKeys).call(selectedKeys, getSelectKeys(item.props.children, value));
    } else {
      var itemValue = getValuePropValue(item);
      var itemKey = item.key;
      if (findIndexInValueBySingleValue(value, itemValue) !== -1 && itemKey) {
        selectedKeys.push(itemKey);
      }
    }
  });
  return selectedKeys;
}
export var UNSELECTABLE_STYLE = {
  userSelect: 'none',
  WebkitUserSelect: 'none'
};
export var UNSELECTABLE_ATTRIBUTE = {
  unselectable: 'on'
};
export function findFirstMenuItem(children) {
  for (var i = 0; i < children.length; i++) {
    var child = children[i];
    if (child.type.isMenuItemGroup) {
      var found = findFirstMenuItem(child.props.children);
      if (found) {
        return found;
      }
    } else if (!child.props.disabled) {
      return child;
    }
  }
  return null;
}
export function includesSeparators(string, separators) {
  for (var i = 0; i < separators.length; ++i) {
    if (_lastIndexOfInstanceProperty(string).call(string, separators[i]) > 0) {
      return true;
    }
  }
  return false;
}
export function splitBySeparators(string, separators) {
  var _context4;
  var reg = new RegExp("[".concat(separators.join(), "]"));
  return _filterInstanceProperty(_context4 = string.split(reg)).call(_context4, function (token) {
    return token;
  });
}
export function defaultFilterFn(input, child) {
  var _context5;
  if (child.props.disabled) {
    return false;
  }
  var value = toArray(getPropValue(child, this.props.optionFilterProp)).join('');
  return _indexOfInstanceProperty(_context5 = value.toLowerCase()).call(_context5, input.toLowerCase()) > -1;
}
export function validateOptionValue(value, props) {
  if (isSingleMode(props) || isMultiple(props)) {
    return;
  }
  if (typeof value !== 'string') {
    throw new Error("Invalid `value` of type `".concat(_typeof(value), "` supplied to Option, ") + "expected `string` when `tags/combobox` is `true`.");
  }
}
export function saveRef(instance, name) {
  return function (node) {
    instance[name] = node;
  };
}