import _classCallCheck from "@babel/runtime-corejs3/helpers/classCallCheck";
import _createClass from "@babel/runtime-corejs3/helpers/createClass";
import _Object$assign from "@babel/runtime-corejs3/core-js-stable/object/assign";
import _bindInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/bind";
import _filterInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/filter";
import Overlay from './core/overlay';
import Element from './core/element';
import Popover from './core/popover';
import { CLASS_CLOSE_BTN, CLASS_SKIP_BTN, CLASS_NEXT_STEP_BTN, CLASS_PREV_STEP_BTN, ESC_KEY_CODE, ID_POPOVER, LEFT_KEY_CODE, OVERLAY_OPACITY, OVERLAY_PADDING, RIGHT_KEY_CODE, SHOULD_ANIMATE_OVERLAY, SHOULD_SHOW_COUNTER, SHOULD_OUTSIDE_CLICK_CLOSE, SHOULD_OUTSIDE_CLICK_NEXT, ALLOW_KEYBOARD_CONTROL } from './common/constants';
import Stage from './core/stage';
import { isDomElement } from './common/utils';
import './index.less';
/**
 * Plugin class that drives the plugin
 */
var Driver = /*#__PURE__*/function () {
  /**
   * @param {Object} options
   */
  function Driver() {
    var _context, _context2, _context3, _context4, _context5, _context6, _context7;
    var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    _classCallCheck(this, Driver);
    this.options = _Object$assign({
      animate: SHOULD_ANIMATE_OVERLAY,
      opacity: OVERLAY_OPACITY,
      padding: OVERLAY_PADDING,
      scrollIntoViewOptions: null,
      counter: SHOULD_SHOW_COUNTER,
      allowClose: SHOULD_OUTSIDE_CLICK_CLOSE,
      keyboardControl: ALLOW_KEYBOARD_CONTROL,
      overlayClickNext: SHOULD_OUTSIDE_CLICK_NEXT,
      stageBackground: '#ffffff',
      onHighlightStarted: function onHighlightStarted() {
        return null;
      },
      onHighlighted: function onHighlighted() {
        return null;
      },
      onDeselected: function onDeselected() {
        return null;
      },
      onReset: function onReset() {
        return null;
      },
      onNext: function onNext() {
        return null;
      },
      onPrevious: function onPrevious() {
        return null;
      }
    }, options);
    this.document = document;
    this.window = window;
    this.isActivated = false;
    this.steps = []; // steps to be presented if any
    this.currentStep = 0; // index for the currently highlighted step
    this.currentMovePrevented = false; // If the current move was prevented
    this.overlay = new Overlay(this.options, window, document);
    this.onResize = _bindInstanceProperty(_context = this.onResize).call(_context, this);
    this.onKeyUp = _bindInstanceProperty(_context2 = this.onKeyUp).call(_context2, this);
    this.onClick = _bindInstanceProperty(_context3 = this.onClick).call(_context3, this);
    this.moveNext = _bindInstanceProperty(_context4 = this.moveNext).call(_context4, this);
    this.movePrevious = _bindInstanceProperty(_context5 = this.movePrevious).call(_context5, this);
    this.preventMove = _bindInstanceProperty(_context6 = this.preventMove).call(_context6, this);
    // Event bindings
    _bindInstanceProperty(_context7 = this).call(_context7);
  }
  /**
   * Getter for steps property
   * @public
   */
  _createClass(Driver, [{
    key: "getSteps",
    value: function getSteps() {
      return this.steps;
    }
    /**
     * Setter for steps property
     * @param steps
     * @public
     */
  }, {
    key: "setSteps",
    value: function setSteps(steps) {
      this.steps = steps;
    }
    /**
     * Binds any DOM events listeners
     * @todo: add throttling in all the listeners
     * @private
     */
  }, {
    key: "bind",
    value: function bind() {
      this.window.addEventListener('resize', this.onResize, false);
      this.window.addEventListener('keyup', this.onKeyUp, false);
      // Binding both touch and click results in popup getting shown and then immediately get hidden.
      // Adding the check to not bind the click event if the touch is supported i.e. on mobile devices
      // Issue: https://github.com/kamranahmedse/driver.js/issues/150
      if (!('ontouchstart' in document.documentElement)) {
        this.window.addEventListener('click', this.onClick, false);
      }
      this.window.addEventListener('touchstart', this.onClick, false);
    }
    /**
     * Removes the popover if clicked outside the highlighted element
     * or outside the
     * @param e
     * @private
     */
  }, {
    key: "onClick",
    value: function onClick(e) {
      if (!this.isActivated || !this.hasHighlightedElement()) {
        return;
      }
      // Stop the event propagation on click/tap. `onClick` handles
      // both touch and click events – which on some browsers causes
      // the click to close the tour
      e.stopPropagation();
      var highlightedElement = this.overlay.getHighlightedElement();
      var popover = this.document.getElementById(ID_POPOVER);
      var clickedHighlightedElement = highlightedElement.node.contains(e.target);
      var clickedPopover = popover && popover.contains(e.target);
      // Perform the 'Next' operation when clicked outside the highlighted element
      if (!clickedHighlightedElement && !clickedPopover && this.options.overlayClickNext) {
        this.handleNext();
        return;
      }
      // Remove the overlay If clicked outside the highlighted element
      if (!clickedHighlightedElement && !clickedPopover && this.options.allowClose) {
        this.reset();
        return;
      }
      var nextClicked = e.target.classList.contains(CLASS_NEXT_STEP_BTN);
      var prevClicked = e.target.classList.contains(CLASS_PREV_STEP_BTN);
      var closeClicked = e.target.classList.contains(CLASS_CLOSE_BTN) || e.target.classList.contains(CLASS_SKIP_BTN);
      if (closeClicked) {
        this.reset();
        return;
      }
      if (nextClicked) {
        this.handleNext();
      } else if (prevClicked) {
        this.handlePrevious();
      }
    }
    /**
     * Handler for the onResize DOM event
     * Makes sure highlighted element stays at valid position
     * @private
     */
  }, {
    key: "onResize",
    value: function onResize() {
      if (!this.isActivated) {
        return;
      }
      this.refresh();
    }
    /**
     * Refreshes and repositions the popover and the overlay
     */
  }, {
    key: "refresh",
    value: function refresh() {
      this.overlay.refresh();
    }
    /**
     * Clears the overlay on escape key process
     * @param event
     * @private
     */
  }, {
    key: "onKeyUp",
    value: function onKeyUp(event) {
      // If driver is not active or keyboard control is disabled
      if (!this.isActivated || !this.options.keyboardControl) {
        return;
      }
      // If escape was pressed and it is allowed to click outside to close
      if (event.keyCode === ESC_KEY_CODE) {
        this.reset();
        return;
      }
      // If there is no highlighted element or there is a highlighted element
      // without popover or if the popover does not allow buttons - ignore
      var highlightedElement = this.getHighlightedElement();
      if (!highlightedElement || !highlightedElement.popover) {
        return;
      }
      if (event.keyCode === RIGHT_KEY_CODE) {
        this.handleNext();
      } else if (event.keyCode === LEFT_KEY_CODE) {
        this.handlePrevious();
      }
    }
    /**
     * Moves to the previous step if possible
     * otherwise resets the overlay
     * @public
     */
  }, {
    key: "movePrevious",
    value: function movePrevious() {
      var previousStep = this.steps[this.currentStep - 1];
      if (!previousStep) {
        // this.reset();
        return;
      }
      this.overlay.highlight(previousStep);
      this.currentStep -= 1;
    }
    /**
     * Prevents the current move. Useful in `onNext` if you want to
     * perform some asynchronous task and manually move to next step
     * @public
     */
  }, {
    key: "preventMove",
    value: function preventMove() {
      this.currentMovePrevented = true;
    }
    /**
     * Handles the internal "move to next" event
     * @private
     */
  }, {
    key: "handleNext",
    value: function handleNext() {
      this.currentMovePrevented = false;
      // Call the bound `onNext` handler if available
      var currentStep = this.steps[this.currentStep];
      if (currentStep && currentStep.options && currentStep.options.onNext) {
        currentStep.options.onNext(this.overlay.highlightedElement);
      }
      if (this.currentMovePrevented) {
        return;
      }
      this.moveNext();
    }
    /**
     * Handles the internal "move to previous" event
     * @private
     */
  }, {
    key: "handlePrevious",
    value: function handlePrevious() {
      this.currentMovePrevented = false;
      // Call the bound `onPrevious` handler if available
      var currentStep = this.steps[this.currentStep];
      if (currentStep && currentStep.options && currentStep.options.onPrevious) {
        currentStep.options.onPrevious(this.overlay.highlightedElement);
      }
      if (this.currentMovePrevented) {
        return;
      }
      this.movePrevious();
    }
    /**
     * Moves to the next step if possible
     * otherwise resets the overlay
     * @public
     */
  }, {
    key: "moveNext",
    value: function moveNext() {
      var nextStep = this.steps[this.currentStep + 1];
      if (!nextStep) {
        this.reset();
        return;
      }
      this.overlay.highlight(nextStep);
      this.currentStep += 1;
    }
    /**
     * @returns {boolean}
     * @public
     */
  }, {
    key: "hasNextStep",
    value: function hasNextStep() {
      return !!this.steps[this.currentStep + 1];
    }
    /**
     * @returns {boolean}
     * @public
     */
  }, {
    key: "hasPreviousStep",
    value: function hasPreviousStep() {
      return !!this.steps[this.currentStep - 1];
    }
    /**
     * Resets the steps if any and clears the overlay
     * @param {boolean} immediate
     * @public
     */
  }, {
    key: "reset",
    value: function reset() {
      var immediate = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
      this.currentStep = 0;
      this.isActivated = false;
      this.overlay.clear(immediate);
    }
    /**
     * Checks if there is any highlighted element or not
     * @returns {boolean}
     * @public
     */
  }, {
    key: "hasHighlightedElement",
    value: function hasHighlightedElement() {
      var highlightedElement = this.overlay.getHighlightedElement();
      return highlightedElement && highlightedElement.node;
    }
    /**
     * Gets the currently highlighted element in overlay
     * @returns {Element}
     * @public
     */
  }, {
    key: "getHighlightedElement",
    value: function getHighlightedElement() {
      return this.overlay.getHighlightedElement();
    }
    /**
     * Gets the element that was highlighted before currently highlighted element
     * @returns {Element}
     * @public
     */
  }, {
    key: "getLastHighlightedElement",
    value: function getLastHighlightedElement() {
      return this.overlay.getLastHighlightedElement();
    }
    /**
     * Defines steps to be highlighted
     * @param {array} steps
     * @public
     */
  }, {
    key: "defineSteps",
    value: function defineSteps(steps) {
      this.steps = [];
      for (var counter = 0; counter < steps.length; counter++) {
        var element = this.prepareElementFromStep(steps[counter], steps, counter);
        if (!element) {
          continue;
        }
        this.steps.push(element);
      }
    }
    /**
     * Prepares the step received from the user and returns an instance
     * of Element
     *
     * @param currentStep Step that is being prepared
     * @param allSteps  List of all the steps
     * @param index Index of the current step
     * @returns {null|Element}
     * @private
     */
  }, {
    key: "prepareElementFromStep",
    value: function prepareElementFromStep(currentStep) {
      var allSteps = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
      var index = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;
      var elementOptions = _Object$assign({}, this.options);
      var querySelector = currentStep;
      // If the `currentStep` is step definition
      // then grab the options and element from the definition
      var isStepDefinition = typeof currentStep !== 'string' && !isDomElement(currentStep);
      if (!currentStep || isStepDefinition && !currentStep.element) {
        throw new Error("Element is required in step ".concat(index));
      }
      if (isStepDefinition) {
        querySelector = currentStep.element;
        elementOptions = _Object$assign(_Object$assign({}, this.options), currentStep);
      }
      // If the given element is a query selector or a DOM element?
      var domElement = isDomElement(querySelector) ? querySelector : this.document.querySelector(querySelector);
      if (!domElement) {
        // console.warn(`Element to highlight ${querySelector} not found`);
        return null;
      }
      var popover = null;
      if (elementOptions.popover && elementOptions.popover.title) {
        var _context8;
        var mergedClassNames = _filterInstanceProperty(_context8 = [this.options.className, elementOptions.popover.className]).call(_context8, function (c) {
          return c;
        }).join(' ');
        var popoverOptions = _Object$assign(_Object$assign(_Object$assign({}, elementOptions), elementOptions.popover), {
          className: mergedClassNames,
          totalCount: allSteps.length,
          currentIndex: index,
          isFirst: index === 0,
          isLast: allSteps.length === 0 || index === allSteps.length - 1
        });
        popover = new Popover(popoverOptions, this.window, this.document);
      }
      var stageOptions = _Object$assign(_Object$assign({}, elementOptions), {
        totalCount: allSteps.length,
        currentIndex: index
      });
      var stage = new Stage(stageOptions, this.window, this.document);
      return new Element({
        node: domElement,
        options: elementOptions,
        popover: popover,
        stage: stage,
        overlay: this.overlay,
        window: this.window,
        document: this.document
      });
    }
    /**
     * Initiates highlighting steps from first step
     * @param {number} index at which highlight is to be started
     * @public
     */
  }, {
    key: "start",
    value: function start() {
      var index = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;
      if (!this.steps || this.steps.length === 0) {
        throw new Error('There are no steps defined to iterate');
      }
      this.isActivated = true;
      this.currentStep = index;
      this.overlay.highlight(this.steps[index]);
    }
    /**
     * Highlights the given element
     * @param {string|{element: string, popover: {}}} selector Query selector or a step definition
     * @public
     */
  }, {
    key: "highlight",
    value: function highlight(selector) {
      this.isActivated = true;
      var element = this.prepareElementFromStep(selector);
      if (!element) {
        return;
      }
      this.overlay.highlight(element);
    }
  }]);
  return Driver;
}();
export { Driver as default };