import * as React from 'react';
import { Divider, ItemGroup } from './src';
import SubMenu from './SubMenu';
import Item from './MenuItem';
import { SiderContext } from '../Layout/Sider';
import './style/index.less';
export interface SelectParam {
    key: string;
    keyPath: Array<string>;
    item: any;
    domEvent: any;
    selectedKeys: Array<string>;
}
export interface ClickParam {
    key: string;
    keyPath: Array<string>;
    item: any;
    domEvent: any;
}
export declare type MenuMode = 'vertical' | 'vertical-left' | 'vertical-right' | 'horizontal' | 'inline';
export declare type MenuTheme = 'light' | 'dark';
export interface MenuProps {
    id?: string;
    theme?: MenuTheme;
    mode?: MenuMode;
    selectable?: boolean;
    selectedKeys?: Array<string>;
    defaultSelectedKeys?: Array<string>;
    openKeys?: Array<string>;
    defaultOpenKeys?: Array<string>;
    onOpenChange?: (openKeys: string[]) => void;
    onSelect?: (param: SelectParam) => void;
    onDeselect?: (param: SelectParam) => void;
    onClick?: (param: ClickParam) => void;
    style?: React.CSSProperties;
    openAnimation?: string | Record<string, any>;
    openTransitionName?: string | Record<string, any>;
    className?: string;
    prefixCls?: string;
    multiple?: boolean;
    inlineIndent?: number;
    inlineCollapsed?: boolean;
    subMenuCloseDelay?: number;
    subMenuOpenDelay?: number;
    getPopupContainer?: (triggerNode: HTMLElement) => HTMLElement;
    focusable?: boolean;
}
export interface MenuState {
    openKeys: string[];
}
declare class Menu extends React.Component<MenuProps, MenuState> {
    static Divider: typeof Divider;
    static Item: typeof Item;
    static SubMenu: typeof SubMenu;
    static ItemGroup: typeof ItemGroup;
    static defaultProps: Partial<MenuProps>;
    static childContextTypes: {
        inlineCollapsed: any;
        menuTheme: any;
    };
    static contextTypes: {
        siderCollapsed: any;
        collapsedWidth: any;
    };
    siderCollapsed: boolean;
    switchModeFromInline: boolean;
    leaveAnimationExecutedWhenInlineCollapsed: boolean;
    inlineOpenKeys: string[];
    static getDerivedStateFromProps(nextProps: MenuProps, prevState: MenuState): {};
    constructor(props: MenuProps, context: SiderContext);
    componentDidUpdate(prevProps: any, prevState: any, snapshot: any): void;
    getChildContext(): {
        inlineCollapsed: any;
        menuTheme: MenuTheme;
    };
    handleClick: (e: ClickParam) => void;
    handleOpenChange: (openKeys: string[]) => void;
    setOpenKeys(openKeys: string[]): void;
    getRealMenuMode(): MenuMode;
    getInlineCollapsed(): any;
    getMenuOpenAnimation(menuMode: MenuMode): string | Record<string, any>;
    render(): JSX.Element;
}
export default Menu;
