import _slicedToArray from "@babel/runtime-corejs3/helpers/slicedToArray";
import _classCallCheck from "@babel/runtime-corejs3/helpers/classCallCheck";
import _createClass from "@babel/runtime-corejs3/helpers/createClass";
import _Object$assign from "@babel/runtime-corejs3/core-js-stable/object/assign";
import _forEachInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/for-each";
import positions from 'position.js';
import '../../style/table.less';
var QuillEvents;
(function (QuillEvents) {
  QuillEvents["EDITOR_CHANGE"] = "editor-change";
  QuillEvents["SCROLL_BEFORE_UPDATE"] = "scroll-before-update";
  QuillEvents["SCROLL_BLOT_MOUNT"] = "scroll-blot-mount";
  QuillEvents["SCROLL_BLOT_UNMOUNT"] = "scroll-blot-unmount";
  QuillEvents["SCROLL_OPTIMIZE"] = "scroll-optimize";
  QuillEvents["SCROLL_UPDATE"] = "scroll-update";
  QuillEvents["SELECTION_CHANGE"] = "selection-change";
  QuillEvents["TEXT_CHANGE"] = "text-change";
})(QuillEvents || (QuillEvents = {}));
var QuillSources;
(function (QuillSources) {
  QuillSources["API"] = "api";
  QuillSources["SILENT"] = "silent";
  QuillSources["USER"] = "user";
})(QuillSources || (QuillSources = {}));
export var isTable = function isTable(quill, range) {
  if (!range) {
    range = quill.getSelection();
  }
  if (!range) {
    return false;
  }
  var formats = quill.getFormat(range.index);
  return !!(formats['table'] && !range.length);
};
var TableUI = /*#__PURE__*/function () {
  function TableUI(quill, options) {
    var _this = this;
    _classCallCheck(this, TableUI);
    this.DEFAULTS = {
      Locale: {}
    };
    this.editorChangeHandler = function (type, range, oldRange, source) {
      if (type === QuillEvents.SELECTION_CHANGE) {
        _this.detectButton(range);
      }
    };
    this.contextMenuHandler = function (evt) {
      if (!isTable(_this.quill)) {
        return true;
      }
      evt.preventDefault();
      _this.showMenu();
    };
    this.scrollHandler = function (evt) {
      evt.preventDefault();
      _this.hideToggle();
      _this.hideMenu();
    };
    this.toggleClickHandler = function (e) {
      _this.toggleMenu();
      e.preventDefault();
      e.stopPropagation();
    };
    this.docClickHandler = function () {
      return _this.hideMenu;
    };
    this.quill = quill;
    this.options = _Object$assign(_Object$assign({}, this.DEFAULTS), options);
    this.table = quill.getModule('table');
    if (!this.table) {
      console.error('"table" module not found');
      return;
    }
    var Locale = this.options.Locale;
    this.menuItems = [{
      title: Locale.insertColRight,
      icon: 'fishdicon-youlie',
      handler: function handler() {
        _this.table.insertColumnRight();
      }
    }, {
      title: Locale.insertColLeft,
      icon: 'fishdicon-zuolie',
      handler: function handler() {
        _this.table.insertColumnLeft();
      }
    }, {
      title: Locale.insertRowAbove,
      icon: 'fishdicon-shanghang',
      handler: function handler() {
        _this.table.insertRowAbove();
      }
    }, {
      title: Locale.insertRowBelow,
      icon: 'fishdicon-xiahang',
      handler: function handler() {
        _this.table.insertRowBelow();
      }
    }, {
      title: Locale.deleteCol,
      icon: 'fishdicon-shanlie',
      handler: function handler() {
        _this.table.deleteColumn();
      }
    }, {
      title: Locale.deleteRow,
      icon: 'fishdicon-shanhang',
      handler: function handler() {
        _this.table.deleteRow();
      }
    }, {
      title: Locale.deleteTable,
      icon: 'fishdicon-delete-line',
      handler: function handler() {
        _this.table.deleteTable();
      }
    }];
    this.toggle = quill.addContainer('ql-table-toggle');
    this.toggle.classList.add('fishdicon', 'fishdicon-gengduo2', 'ql-table-toggle_hidden');
    this.toggle.addEventListener('click', this.toggleClickHandler);
    this.quill.on(QuillEvents.EDITOR_CHANGE, this.editorChangeHandler);
    this.quill.root.addEventListener('contextmenu', this.contextMenuHandler);
    // fix: toggle and menu separate from the table
    this.quill.root.addEventListener('scroll', this.scrollHandler);
  }
  /*
  getColCount(range: Range = null) {
    if (!range) {
      range = this.quill.getSelection();
    }
    if (!range) {
      return 0;
    }
    const [table] = this.table.getTable(range);
    if (!table) {
      return 0;
    }
       const maxColumns = table.rows().reduce((max, row) => {
      return Math.max(row.children.length, max);
    }, 0);
    return maxColumns;
  }
     getRowCount(range: Range = null) {
    if (!range) {
      range = this.quill.getSelection();
    }
    if (!range) {
      return 0;
    }
    const [table] = this.table.getTable(range);
    if (!table) {
      return 0;
    }
       return table.rows().length;
  }
  */
  _createClass(TableUI, [{
    key: "showMenu",
    value: function showMenu() {
      var _context,
        _this2 = this;
      this.hideMenu();
      this.menu = this.quill.addContainer('ql-table-menu');
      _forEachInstanceProperty(_context = this.menuItems).call(_context, function (it) {
        _this2.menu.appendChild(_this2.createMenuItem(it));
      });
      // set menu position
      var position = positions(this.menu, this.toggle, {
        popup: 'top-left',
        anchor: 'bottom-left'
      }, {
        offsetParent: this.quill.container
      });
      // dynamic adjustment when the menu cannot be displayed completely
      if (this.quill.container.clientWidth - position.left < this.menu.clientWidth) {
        this.menu.style.left = "".concat(this.quill.container.clientWidth - this.menu.clientWidth, "px");
      } else {
        this.menu.style.left = "".concat(position.left, "px");
      }
      if (this.quill.container.clientHeight - position.top < this.menu.clientHeight) {
        this.menu.style.top = "".concat(this.quill.container.clientHeight - this.menu.clientHeight, "px");
      } else {
        this.menu.style.top = "".concat(position.top, "px");
      }
      document.addEventListener('click', this.docClickHandler);
    }
  }, {
    key: "hideMenu",
    value: function hideMenu() {
      if (this.menu) {
        this.menu.remove();
        this.menu = null;
        document.removeEventListener('click', this.docClickHandler);
      }
    }
  }, {
    key: "createMenuItem",
    value: function createMenuItem(item) {
      var _this3 = this;
      var node = document.createElement('div');
      node.classList.add('ql-table-menu__item');
      node.title = item.title;
      var iconSpan = document.createElement('span');
      iconSpan.classList.add('ql-table-menu__item-icon', 'fishdicon', item.icon);
      node.appendChild(iconSpan);
      node.addEventListener('click', function (e) {
        e.preventDefault();
        e.stopPropagation();
        _this3.quill.focus();
        item.handler();
        _this3.hideMenu();
        _this3.detectButton(_this3.quill.getSelection());
      }, false);
      return node;
    }
  }, {
    key: "detectButton",
    value: function detectButton(range) {
      if (range == null) {
        return;
      }
      var show = isTable(this.quill, range);
      if (show) {
        var _this$quill$getLine = this.quill.getLine(range.index),
          _this$quill$getLine2 = _slicedToArray(_this$quill$getLine, 2),
          cell = _this$quill$getLine2[0],
          offset = _this$quill$getLine2[1];
        var containerBounds = this.quill.container.getBoundingClientRect();
        var bounds = cell.domNode.getBoundingClientRect();
        bounds = {
          bottom: bounds.bottom - containerBounds.top,
          height: bounds.height,
          left: bounds.left - containerBounds.left,
          right: bounds.right - containerBounds.left,
          top: bounds.top - containerBounds.top + this.quill.container.scrollTop,
          width: bounds.width
        };
        this.showToggle(bounds);
      } else {
        this.hideToggle();
      }
      this.hideMenu();
    }
  }, {
    key: "showToggle",
    value: function showToggle(position) {
      this.position = position;
      this.toggle.classList.remove('ql-table-toggle_hidden');
      this.toggle.style.top = "".concat(position.top, "px");
      this.toggle.style.left = "".concat(position.left, "px");
    }
  }, {
    key: "hideToggle",
    value: function hideToggle() {
      this.toggle && this.toggle.classList.add('ql-table-toggle_hidden');
    }
  }, {
    key: "toggleMenu",
    value: function toggleMenu() {
      if (this.menu) {
        this.hideToggle();
      } else {
        this.showMenu();
      }
    }
  }, {
    key: "destroy",
    value: function destroy() {
      this.hideMenu();
      this.quill.off(QuillEvents.EDITOR_CHANGE, this.editorChangeHandler);
      this.quill.root.removeEventListener('contextmenu', this.contextMenuHandler);
      this.quill.root.removeEventListener('scroll', this.scrollHandler);
      this.toggle.removeEventListener('click', this.toggleClickHandler);
      this.toggle.remove();
      this.toggle = null;
      this.options = this.DEFAULTS;
      this.menu = null;
      this.table = null;
      this.quill = null;
    }
  }]);
  return TableUI;
}();
export { TableUI as default };