import _toArray from "@babel/runtime-corejs3/helpers/toArray";
import _defineProperty from "@babel/runtime-corejs3/helpers/defineProperty";
import _typeof from "@babel/runtime-corejs3/helpers/typeof";
import _slicedToArray from "@babel/runtime-corejs3/helpers/slicedToArray";
import _classCallCheck from "@babel/runtime-corejs3/helpers/classCallCheck";
import _createClass from "@babel/runtime-corejs3/helpers/createClass";
import _reduceInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/reduce";
import _endsWithInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/ends-with";
import _Object$keys from "@babel/runtime-corejs3/core-js-stable/object/keys";
import _forEachInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/for-each";
import _sliceInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/slice";
import _concatInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/concat";
import _Object$assign from "@babel/runtime-corejs3/core-js-stable/object/assign";
import _mapInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/map";
import _filterInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/filter";
import _findInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/find";
import _Array$isArray from "@babel/runtime-corejs3/core-js-stable/array/is-array";
import _indexOfInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/index-of";
import cloneDeep from 'lodash/cloneDeep';
import isEqual from 'lodash/isEqual';
import merge from 'lodash/merge';
import Delta, { AttributeMap, Op } from 'quill-delta';
import { LeafBlot, Scope } from 'parchment';
import { Range } from './selection';
import CursorBlot from '../blots/cursor';
import Block, { BlockEmbed, bubbleFormats } from '../blots/block';
import Break from '../blots/break';
import TextBlot, { escapeText } from '../blots/text';
var ASCII = /^[ -~]*$/;
var Editor = /*#__PURE__*/function () {
  function Editor(scroll) {
    _classCallCheck(this, Editor);
    this.scroll = scroll;
    this.delta = this.getDelta();
  }
  _createClass(Editor, [{
    key: "applyDelta",
    value: function applyDelta(delta) {
      var _this = this;
      this.scroll.update();
      var scrollLength = this.scroll.length();
      this.scroll.batchStart();
      var normalizedDelta = normalizeDelta(delta);
      var deleteDelta = new Delta();
      _reduceInstanceProperty(normalizedDelta).call(normalizedDelta, function (index, op) {
        var _context;
        var length = Op.length(op);
        var attributes = op.attributes || {};
        var addedNewline = false;
        if (op.insert != null) {
          deleteDelta.retain(length);
          if (typeof op.insert === 'string') {
            var text = op.insert;
            addedNewline = !_endsWithInstanceProperty(text).call(text, '\n') && (scrollLength <= index || _this.scroll.descendant(BlockEmbed, index)[0]);
            _this.scroll.insertAt(index, text);
            var _this$scroll$line = _this.scroll.line(index),
              _this$scroll$line2 = _slicedToArray(_this$scroll$line, 2),
              line = _this$scroll$line2[0],
              offset = _this$scroll$line2[1];
            var formats = merge({}, bubbleFormats(line));
            if (line instanceof Block) {
              var _line$descendant = line.descendant(LeafBlot, offset),
                _line$descendant2 = _slicedToArray(_line$descendant, 1),
                leaf = _line$descendant2[0];
              formats = merge(formats, bubbleFormats(leaf));
            }
            attributes = AttributeMap.diff(formats, attributes) || {};
          } else if (_typeof(op.insert) === 'object') {
            var key = _Object$keys(op.insert)[0]; // There should only be one key
            if (key == null) return index;
            addedNewline = _this.scroll.query(key, Scope.INLINE) != null && (scrollLength <= index || _this.scroll.descendant(BlockEmbed, index)[0]);
            _this.scroll.insertAt(index, key, op.insert[key]);
          }
          scrollLength += length;
        } else {
          deleteDelta.push(op);
        }
        _forEachInstanceProperty(_context = _Object$keys(attributes)).call(_context, function (name) {
          _this.scroll.formatAt(index, length, name, attributes[name]);
        });
        var addedLength = addedNewline ? 1 : 0;
        scrollLength += addedLength;
        deleteDelta["delete"](addedLength);
        return index + length + addedLength;
      }, 0);
      _reduceInstanceProperty(deleteDelta).call(deleteDelta, function (index, op) {
        if (typeof op["delete"] === 'number') {
          _this.scroll.deleteAt(index, op["delete"]);
          return index;
        }
        return index + Op.length(op);
      }, 0);
      this.scroll.batchEnd();
      this.scroll.optimize();
      return this.update(normalizedDelta);
    }
  }, {
    key: "deleteText",
    value: function deleteText(index, length) {
      this.scroll.deleteAt(index, length);
      return this.update(new Delta().retain(index)["delete"](length));
    }
  }, {
    key: "formatLine",
    value: function formatLine(index, length) {
      var _context2,
        _this2 = this;
      var formats = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
      this.scroll.update();
      _forEachInstanceProperty(_context2 = _Object$keys(formats)).call(_context2, function (format) {
        var _context3;
        _forEachInstanceProperty(_context3 = _this2.scroll.lines(index, Math.max(length, 1))).call(_context3, function (line) {
          line.format(format, formats[format]);
        });
      });
      this.scroll.optimize();
      var delta = new Delta().retain(index).retain(length, cloneDeep(formats));
      return this.update(delta);
    }
  }, {
    key: "formatText",
    value: function formatText(index, length) {
      var _context4,
        _this3 = this;
      var formats = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
      _forEachInstanceProperty(_context4 = _Object$keys(formats)).call(_context4, function (format) {
        _this3.scroll.formatAt(index, length, format, formats[format]);
      });
      var delta = new Delta().retain(index).retain(length, cloneDeep(formats));
      return this.update(delta);
    }
  }, {
    key: "getContents",
    value: function getContents(index, length) {
      var _context5;
      return _sliceInstanceProperty(_context5 = this.delta).call(_context5, index, index + length);
    }
  }, {
    key: "getDelta",
    value: function getDelta() {
      var _context6;
      return _reduceInstanceProperty(_context6 = this.scroll.lines()).call(_context6, function (delta, line) {
        return _concatInstanceProperty(delta).call(delta, line.delta());
      }, new Delta());
    }
  }, {
    key: "getFormat",
    value: function getFormat(index) {
      var _context8;
      var length = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
      var lines = [];
      var leaves = [];
      if (length === 0) {
        var _context7;
        _forEachInstanceProperty(_context7 = this.scroll.path(index)).call(_context7, function (path) {
          var _path = _slicedToArray(path, 1),
            blot = _path[0];
          if (blot instanceof Block) {
            lines.push(blot);
          } else if (blot instanceof LeafBlot) {
            leaves.push(blot);
          }
        });
      } else {
        lines = this.scroll.lines(index, length);
        leaves = this.scroll.descendants(LeafBlot, index, length);
      }
      var _map = _mapInstanceProperty(_context8 = [lines, leaves]).call(_context8, function (blots) {
        if (blots.length === 0) return {};
        var formats = bubbleFormats(blots.shift());
        while (_Object$keys(formats).length > 0) {
          var blot = blots.shift();
          if (blot == null) return formats;
          formats = combineFormats(bubbleFormats(blot), formats);
        }
        return formats;
      });
      var _map2 = _slicedToArray(_map, 2);
      lines = _map2[0];
      leaves = _map2[1];
      return _Object$assign(_Object$assign({}, lines), leaves);
    }
  }, {
    key: "getHTML",
    value: function getHTML(index, length) {
      var _this$scroll$line3 = this.scroll.line(index),
        _this$scroll$line4 = _slicedToArray(_this$scroll$line3, 2),
        line = _this$scroll$line4[0],
        lineOffset = _this$scroll$line4[1];
      if (line.length() >= lineOffset + length) {
        return convertHTML(line, lineOffset, length, true);
      }
      return convertHTML(this.scroll, index, length, true);
    }
  }, {
    key: "getText",
    value: function getText(index, length) {
      var _context9, _context10;
      return _mapInstanceProperty(_context9 = _filterInstanceProperty(_context10 = this.getContents(index, length)).call(_context10, function (op) {
        return typeof op.insert === 'string';
      })).call(_context9, function (op) {
        return op.insert;
      }).join('');
    }
  }, {
    key: "insertEmbed",
    value: function insertEmbed(index, embed, value) {
      this.scroll.insertAt(index, embed, value);
      return this.update(new Delta().retain(index).insert(_defineProperty({}, embed, value)));
    }
  }, {
    key: "insertText",
    value: function insertText(index, text) {
      var _context11,
        _this4 = this;
      var formats = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
      text = text.replace(/\r\n/g, '\n').replace(/\r/g, '\n');
      this.scroll.insertAt(index, text);
      _forEachInstanceProperty(_context11 = _Object$keys(formats)).call(_context11, function (format) {
        _this4.scroll.formatAt(index, text.length, format, formats[format]);
      });
      return this.update(new Delta().retain(index).insert(text, cloneDeep(formats)));
    }
  }, {
    key: "isBlank",
    value: function isBlank() {
      if (this.scroll.children.length === 0) return true;
      if (this.scroll.children.length > 1) return false;
      var block = this.scroll.children.head;
      if (block.statics.blotName !== Block.blotName) return false;
      if (block.children.length > 1) return false;
      return block.children.head instanceof Break;
    }
  }, {
    key: "removeFormat",
    value: function removeFormat(index, length) {
      var _context13, _context14;
      var text = this.getText(index, length);
      var _this$scroll$line5 = this.scroll.line(index + length),
        _this$scroll$line6 = _slicedToArray(_this$scroll$line5, 2),
        line = _this$scroll$line6[0],
        offset = _this$scroll$line6[1];
      var suffixLength = 0;
      var suffix = new Delta();
      if (line != null) {
        var _context12;
        suffixLength = line.length() - offset;
        suffix = _sliceInstanceProperty(_context12 = line.delta()).call(_context12, offset, offset + suffixLength - 1).insert('\n');
      }
      var contents = this.getContents(index, length + suffixLength);
      var diff = contents.diff(_concatInstanceProperty(_context13 = new Delta().insert(text)).call(_context13, suffix));
      var delta = _concatInstanceProperty(_context14 = new Delta().retain(index)).call(_context14, diff);
      return this.applyDelta(delta);
    }
  }, {
    key: "update",
    value: function update(change) {
      var _context15;
      var mutations = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
      var selectionInfo = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : undefined;
      var oldDelta = this.delta;
      if (mutations.length === 1 && mutations[0].type === 'characterData' && mutations[0].target.data.match(ASCII) && _findInstanceProperty(_context15 = this.scroll).call(_context15, mutations[0].target)) {
        var _context16, _context17;
        // Optimization for character changes
        var textBlot = _findInstanceProperty(_context16 = this.scroll).call(_context16, mutations[0].target);
        var formats = bubbleFormats(textBlot);
        var index = textBlot.offset(this.scroll);
        var oldValue = mutations[0].oldValue.replace(CursorBlot.CONTENTS, '');
        var oldText = new Delta().insert(oldValue);
        var newText = new Delta().insert(textBlot.value());
        var relativeSelectionInfo = selectionInfo && {
          oldRange: shiftRange(selectionInfo.oldRange, -index),
          newRange: shiftRange(selectionInfo.newRange, -index)
        };
        var diffDelta = _concatInstanceProperty(_context17 = new Delta().retain(index)).call(_context17, oldText.diff(newText, relativeSelectionInfo));
        change = _reduceInstanceProperty(diffDelta).call(diffDelta, function (delta, op) {
          if (op.insert) {
            return delta.insert(op.insert, formats);
          }
          return delta.push(op);
        }, new Delta());
        this.delta = oldDelta.compose(change);
      } else {
        this.delta = this.getDelta();
        if (!change || !isEqual(oldDelta.compose(change), this.delta)) {
          change = oldDelta.diff(this.delta, selectionInfo);
        }
      }
      return change;
    }
  }]);
  return Editor;
}();
function convertListHTML(items, lastIndent, types) {
  var _context25;
  if (items.length === 0) {
    var _context18;
    var _getListType = getListType(types.pop()),
      _getListType2 = _slicedToArray(_getListType, 1),
      _endTag = _getListType2[0];
    if (lastIndent <= 0) {
      return "</li></".concat(_endTag, ">");
    }
    return _concatInstanceProperty(_context18 = "</li></".concat(_endTag, ">")).call(_context18, convertListHTML([], lastIndent - 1, types));
  }
  var _items = _toArray(items),
    _items$ = _items[0],
    child = _items$.child,
    offset = _items$.offset,
    length = _items$.length,
    indent = _items$.indent,
    type = _items$.type,
    rest = _sliceInstanceProperty(_items).call(_items, 1);
  var _getListType3 = getListType(type),
    _getListType4 = _slicedToArray(_getListType3, 2),
    tag = _getListType4[0],
    attribute = _getListType4[1];
  if (indent > lastIndent) {
    var _context22;
    types.push(type);
    if (indent === lastIndent + 1) {
      var _context19, _context20, _context21;
      return _concatInstanceProperty(_context19 = _concatInstanceProperty(_context20 = _concatInstanceProperty(_context21 = "<".concat(tag, "><li")).call(_context21, attribute, ">")).call(_context20, convertHTML(child, offset, length))).call(_context19, convertListHTML(rest, indent, types));
    }
    return _concatInstanceProperty(_context22 = "<".concat(tag, "><li>")).call(_context22, convertListHTML(items, lastIndent + 1, types));
  }
  var previousType = types[types.length - 1];
  if (indent === lastIndent && type === previousType) {
    var _context23, _context24;
    return _concatInstanceProperty(_context23 = _concatInstanceProperty(_context24 = "</li><li".concat(attribute, ">")).call(_context24, convertHTML(child, offset, length))).call(_context23, convertListHTML(rest, indent, types));
  }
  var _getListType5 = getListType(types.pop()),
    _getListType6 = _slicedToArray(_getListType5, 1),
    endTag = _getListType6[0];
  return _concatInstanceProperty(_context25 = "</li></".concat(endTag, ">")).call(_context25, convertListHTML(items, lastIndent - 1, types));
}
function convertHTML(blot, index, length) {
  var isRoot = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;
  if (typeof blot.html === 'function') {
    return blot.html(index, length);
  }
  if (blot instanceof TextBlot) {
    var _context26;
    return escapeText(_sliceInstanceProperty(_context26 = blot.value()).call(_context26, index, index + length));
  }
  if (blot.children) {
    var _context28, _context29;
    // TODO fix API
    if (blot.statics.blotName === 'list-container') {
      var items = [];
      blot.children.forEachAt(index, length, function (child, offset, childLength) {
        var formats = child.formats();
        items.push({
          child: child,
          offset: offset,
          length: childLength,
          indent: formats.indent || 0,
          type: formats.list
        });
      });
      return convertListHTML(items, -1, []);
    }
    var parts = [];
    blot.children.forEachAt(index, length, function (child, offset, childLength) {
      parts.push(convertHTML(child, offset, childLength));
    });
    if (isRoot || blot.statics.blotName === 'list') {
      return parts.join('');
    }
    var _blot$domNode = blot.domNode,
      outerHTML = _blot$domNode.outerHTML,
      innerHTML = _blot$domNode.innerHTML;
    var _outerHTML$split = outerHTML.split(">".concat(innerHTML, "<")),
      _outerHTML$split2 = _slicedToArray(_outerHTML$split, 2),
      start = _outerHTML$split2[0],
      end = _outerHTML$split2[1];
    // TODO cleanup
    if (start === '<table') {
      var _context27;
      return _concatInstanceProperty(_context27 = "<table style=\"border: 1px solid #000;\">".concat(parts.join(''), "<")).call(_context27, end);
    }
    return _concatInstanceProperty(_context28 = _concatInstanceProperty(_context29 = "".concat(start, ">")).call(_context29, parts.join(''), "<")).call(_context28, end);
  }
  return blot.domNode.outerHTML;
}
function combineFormats(formats, combined) {
  var _context30;
  return _reduceInstanceProperty(_context30 = _Object$keys(combined)).call(_context30, function (merged, name) {
    if (formats[name] == null) return merged;
    if (combined[name] === formats[name]) {
      merged[name] = combined[name];
    } else if (_Array$isArray(combined[name])) {
      var _context31;
      if (_indexOfInstanceProperty(_context31 = combined[name]).call(_context31, formats[name]) < 0) {
        var _context32;
        merged[name] = _concatInstanceProperty(_context32 = combined[name]).call(_context32, [formats[name]]);
      }
    } else {
      merged[name] = [combined[name], formats[name]];
    }
    return merged;
  }, {});
}
function getListType(type) {
  var tag = type === 'ordered' ? 'ol' : 'ul';
  switch (type) {
    case 'checked':
      return [tag, ' data-list="checked"'];
    case 'unchecked':
      return [tag, ' data-list="unchecked"'];
    default:
      return [tag, ''];
  }
}
function normalizeDelta(delta) {
  return _reduceInstanceProperty(delta).call(delta, function (normalizedDelta, op) {
    if (typeof op.insert === 'string') {
      var text = op.insert.replace(/\r\n/g, '\n').replace(/\r/g, '\n');
      return normalizedDelta.insert(text, op.attributes);
    }
    return normalizedDelta.push(op);
  }, new Delta());
}
function shiftRange(_ref, amount) {
  var index = _ref.index,
    length = _ref.length;
  return new Range(index + amount, length);
}
export default Editor;