import _typeof from "@babel/runtime-corejs3/helpers/typeof";
import _indexOfInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/index-of";
import _Object$getOwnPropertySymbols from "@babel/runtime-corejs3/core-js-stable/object/get-own-property-symbols";
import _forEachInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/for-each";
import _sliceInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/slice";
import _spliceInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/splice";
import _concatInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/concat";
import _filterInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/filter";
import _mapInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/map";
import _Array$isArray from "@babel/runtime-corejs3/core-js-stable/array/is-array";
import _Object$assign from "@babel/runtime-corejs3/core-js-stable/object/assign";
import _Object$keys from "@babel/runtime-corejs3/core-js-stable/object/keys";
import _reduceInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/reduce";
var __rest = this && this.__rest || function (s, e) {
  var t = {};
  for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && _indexOfInstanceProperty(e).call(e, p) < 0) t[p] = s[p];
  if (s != null && typeof _Object$getOwnPropertySymbols === "function") for (var i = 0, p = _Object$getOwnPropertySymbols(s); i < p.length; i++) {
    if (_indexOfInstanceProperty(e).call(e, p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i])) t[p[i]] = s[p[i]];
  }
  return t;
};
import React, { Children } from 'react';
import warning from 'warning';
import TreeNode from './TreeNode.js';
var DRAG_SIDE_RANGE = 0.25;
var DRAG_MIN_GAP = 2;
var onlyTreeNodeWarned = false;
export function toArray(children) {
  var ret = [];
  _forEachInstanceProperty(Children).call(Children, children, function (c) {
    ret.push(c);
  });
  return ret;
}
export function warnOnlyTreeNode() {
  if (onlyTreeNodeWarned) return;
  onlyTreeNodeWarned = true;
  warning(false, 'Tree only accept TreeNode as children.');
}
export function arrDel(list, value) {
  var clone = _sliceInstanceProperty(list).call(list);
  var index = _indexOfInstanceProperty(clone).call(clone, value);
  if (index >= 0) {
    _spliceInstanceProperty(clone).call(clone, index, 1);
  }
  return clone;
}
export function arrAdd(list, value) {
  var clone = _sliceInstanceProperty(list).call(list);
  if (_indexOfInstanceProperty(clone).call(clone, value) === -1) {
    clone.push(value);
  }
  return clone;
}
export function posToArr(pos) {
  return pos.split('-');
}
export function getPosition(level, index) {
  var _context;
  return _concatInstanceProperty(_context = "".concat(level, "-")).call(_context, index);
}
export function isTreeNode(node) {
  return node && node.type && node.type.isTreeNode;
}
export function getNodeChildren(children) {
  var _context2;
  return _filterInstanceProperty(_context2 = toArray(children)).call(_context2, isTreeNode);
}
export function isCheckDisabled(node) {
  var _ref = node.props || {},
    disabled = _ref.disabled,
    disableCheckbox = _ref.disableCheckbox;
  return !!(disabled || disableCheckbox);
}
export function traverseTreeNodes(treeNodes, callback) {
  function processNode(node, index, parent) {
    var children = node ? node.props.children : treeNodes;
    var pos = node ? getPosition(parent.pos, index) : 0;
    // Filter children
    var childList = getNodeChildren(children);
    // Process node if is not root
    if (node) {
      var data = {
        node: node,
        index: index,
        pos: pos,
        key: node.key || pos,
        parentPos: parent.node ? parent.pos : null
      };
      callback(data);
    }
    // Process children node
    _forEachInstanceProperty(Children).call(Children, childList, function (subNode, subIndex) {
      processNode(subNode, subIndex, {
        node: node,
        pos: pos
      });
    });
  }
  processNode(null);
}
/**
 * Use `rc-util` `toArray` to get the children list which keeps the key.
 * And return single node if children is only one(This can avoid `key` missing check).
 */
export function mapChildren(children, func) {
  var _context3;
  var list = _mapInstanceProperty(_context3 = toArray(children)).call(_context3, func);
  if (list.length === 1) {
    return list[0];
  }
  return list;
}
export function getDragNodesKeys(treeNodes, node) {
  var _node$props = node.props,
    eventKey = _node$props.eventKey,
    pos = _node$props.pos;
  var dragNodesKeys = [];
  traverseTreeNodes(treeNodes, function (_ref2) {
    var key = _ref2.key;
    dragNodesKeys.push(key);
  });
  dragNodesKeys.push(eventKey || pos);
  return dragNodesKeys;
}
// Only used when drag, not affect SSR.
export function calcDropPosition(event, treeNode) {
  var clientY = event.clientY;
  var _treeNode$selectHandl = treeNode.selectHandle.getBoundingClientRect(),
    top = _treeNode$selectHandl.top,
    bottom = _treeNode$selectHandl.bottom,
    height = _treeNode$selectHandl.height;
  var des = Math.max(height * DRAG_SIDE_RANGE, DRAG_MIN_GAP);
  if (clientY <= top + des) {
    return -1;
  } else if (clientY >= bottom - des) {
    return 1;
  }
  return 0;
}
/**
 * Return selectedKeys according with multiple prop
 * @param selectedKeys
 * @param props
 * @returns [string]
 */
export function calcSelectedKeys(selectedKeys, props) {
  if (!selectedKeys) return undefined;
  var multiple = props.multiple;
  if (multiple) {
    return _sliceInstanceProperty(selectedKeys).call(selectedKeys);
  }
  if (selectedKeys.length) {
    return [selectedKeys[0]];
  }
  return selectedKeys;
}
/**
 * Since React internal will convert key to string,
 * we need do this to avoid `checkStrictly` use number match
 */
function keyListToString(keyList) {
  if (!keyList) return keyList;
  return _mapInstanceProperty(keyList).call(keyList, function (key) {
    return String(key);
  });
}
var internalProcessProps = function internalProcessProps(props) {
  return props;
};
export function convertDataToTree(treeData, processer) {
  if (!treeData) return [];
  var _ref3 = processer || {},
    _ref3$processProps = _ref3.processProps,
    processProps = _ref3$processProps === void 0 ? internalProcessProps : _ref3$processProps;
  var list = _Array$isArray(treeData) ? treeData : [treeData];
  return _mapInstanceProperty(list).call(list, function (item, index) {
    var children = item.children,
      props = __rest(item, ["children"]);
    var childrenNodes = convertDataToTree(children, processer);
    if (item.childCount === undefined) {
      item.childCount = -1; //表示用户未给此数据源赋值，不能执行半选判断逻辑
    }

    return /*#__PURE__*/React.createElement(TreeNode, _Object$assign({
      key: props.key,
      _data: item
    }, processProps(props)), childrenNodes);
  });
}
// TODO: ========================= NEW LOGIC =========================
/**
 * Calculate treeNodes entities. `processTreeEntity` is used for `rc-tree-select`
 * @param treeNodes
 * @param processTreeEntity  User can customize the entity
 */
export function convertTreeToEntities(treeNodes) {
  var _ref4 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
    initWrapper = _ref4.initWrapper,
    processEntity = _ref4.processEntity,
    onProcessFinished = _ref4.onProcessFinished;
  var posEntities = {};
  var keyEntities = {};
  var wrapper = {
    posEntities: posEntities,
    keyEntities: keyEntities
  };
  if (initWrapper) {
    wrapper = initWrapper(wrapper) || wrapper;
  }
  traverseTreeNodes(treeNodes, function (item) {
    var node = item.node,
      index = item.index,
      pos = item.pos,
      key = item.key,
      parentPos = item.parentPos;
    var entity = {
      node: node,
      index: index,
      key: key,
      pos: pos
    };
    posEntities[pos] = entity;
    keyEntities[key] = entity;
    // Fill children
    entity.parent = posEntities[parentPos];
    if (entity.parent) {
      entity.parent.children = entity.parent.children || [];
      entity.parent.children.push(entity);
    }
    if (processEntity) {
      processEntity(entity, wrapper);
    }
  });
  if (onProcessFinished) {
    onProcessFinished(wrapper);
  }
  return wrapper;
}
/**
 * Parse `checkedKeys` to { checkedKeys, halfCheckedKeys } style
 */
export function parseCheckedKeys(keys) {
  if (!keys) {
    return null;
  }
  // Convert keys to object format
  var keyProps;
  if (_Array$isArray(keys)) {
    // [Legacy] Follow the api doc
    keyProps = {
      checkedKeys: keys,
      halfCheckedKeys: undefined
    };
  } else if (_typeof(keys) === 'object') {
    keyProps = {
      checkedKeys: keys.checked || undefined,
      halfCheckedKeys: keys.halfChecked || undefined
    };
  } else {
    warning(false, '`checkedKeys` is not an array or an object');
    return null;
  }
  keyProps.checkedKeys = keyListToString(keyProps.checkedKeys);
  keyProps.halfCheckedKeys = keyListToString(keyProps.halfCheckedKeys);
  return keyProps;
}
/**
 * Conduct check state by the keyList. It will conduct up & from the provided key.
 * If the conduct path reach the disabled or already checked / unchecked node will stop conduct.
 * @param keyList       list of keys
 * @param isCheck       is check the node or not
 * @param keyEntities   parsed by `convertTreeToEntities` function in Tree
 * @param checkStatus   Can pass current checked status for process (usually for uncheck operation)
 * @param checkType     联动类型
 * @returns {{checkedKeys: [], halfCheckedKeys: []}}
 */
export function conductCheck(keyList, isCheck, keyEntities, status, loadData, loadedKeys, checkType) {
  var _context4, _context5, _context11, _context12, _context13;
  var checkedKeys = {};
  var halfCheckedKeys = {}; // Record the key has some child checked (include child half checked)
  var checkStatus = status || {};
  _forEachInstanceProperty(_context4 = checkStatus.checkedKeys || []).call(_context4, function (key) {
    checkedKeys[key] = true;
  });
  _forEachInstanceProperty(_context5 = checkStatus.halfCheckedKeys || []).call(_context5, function (key) {
    halfCheckedKeys[key] = true;
  });
  // Conduct up
  function conductUp(key) {
    var _context6, _context7;
    if (checkedKeys[key] === isCheck) return;
    var entity = keyEntities[key];
    if (!entity) return;
    var children = entity.children,
      parent = entity.parent,
      node = entity.node;
    if (isCheckDisabled(node)) return;
    // Check child node checked status
    var everyChildChecked = true;
    var someChildChecked = false; // Child checked or half checked
    var checkedNum = 0;
    _forEachInstanceProperty(_context6 = _filterInstanceProperty(_context7 = children || []).call(_context7, function (child) {
      return !isCheckDisabled(child.node);
    })).call(_context6, function (child) {
      var childKey = child.key;
      var childChecked = checkedKeys[childKey];
      var childHalfChecked = halfCheckedKeys[childKey];
      if (childChecked) {
        checkedNum++;
      }
      if (childChecked || childHalfChecked) someChildChecked = true;
      if (
      // 取消勾选
      !childChecked
      // 注释以下代码，避免异步加载非搜索状态下上级全选不联动的问题。改为搜索状态下上级节点全选联动，半选不联动。
      // 搜索状态的异步加载模式下，当前父节点key不在已加载的节点中，全选子节点时，不选中父节点
      // || (upperSearchValue && loadData && (!loadedKeys || loadedKeys.indexOf(key) == -1))
      ) {
        everyChildChecked = false;
      }
    });
    // Update checked status
    if (isCheck) {
      if (node.props._data && node.props._data.childCount != undefined) {
        checkedKeys[key] = everyChildChecked && checkedNum >= node.props._data.childCount;
      } else {
        checkedKeys[key] = everyChildChecked;
      }
    } else {
      checkedKeys[key] = false;
    }
    halfCheckedKeys[key] = someChildChecked;
    if (checkType === 'countDown') {
      // 单向联动
      checkedKeys[key] = false;
      halfCheckedKeys[key] = false;
    }
    if (parent) {
      conductUp(parent.key);
    }
  }
  // Conduct down
  function conductDown(key) {
    var _context8;
    if (checkedKeys[key] === isCheck) return;
    var entity = keyEntities[key];
    if (!entity) return;
    var children = entity.children,
      node = entity.node;
    if (isCheckDisabled(node)) return;
    checkedKeys[key] = isCheck;
    _forEachInstanceProperty(_context8 = children || []).call(_context8, function (child) {
      conductDown(child.key);
    });
  }
  function conduct(key) {
    var _context9, _context10;
    var entity = keyEntities[key];
    if (!entity) {
      warning(false, "'".concat(key, "' does not exist in the tree."));
      return;
    }
    var children = entity.children,
      parent = entity.parent,
      node = entity.node;
    checkedKeys[key] = isCheck;
    if (isCheckDisabled(node)) return;
    // Conduct down
    _forEachInstanceProperty(_context9 = _filterInstanceProperty(_context10 = children || []).call(_context10, function (child) {
      return !isCheckDisabled(child.node);
    })).call(_context9, function (child) {
      conductDown(child.key);
    });
    // Conduct up
    if (parent) {
      conductUp(parent.key);
    }
  }
  _forEachInstanceProperty(_context11 = keyList || []).call(_context11, function (key) {
    conduct(key);
  });
  var checkedKeyList = [];
  var halfCheckedKeyList = [];
  // Fill checked list
  _forEachInstanceProperty(_context12 = _Object$keys(checkedKeys)).call(_context12, function (key) {
    if (checkedKeys[key]) {
      checkedKeyList.push(key);
    }
  });
  // Fill half checked list
  _forEachInstanceProperty(_context13 = _Object$keys(halfCheckedKeys)).call(_context13, function (key) {
    if (!checkedKeys[key] && halfCheckedKeys[key]) {
      halfCheckedKeyList.push(key);
    }
  });
  return {
    checkedKeys: checkedKeyList,
    halfCheckedKeys: halfCheckedKeyList
  };
}
export function conductLoad(keyList, isCheck, keyEntities, status) {
  var _context18, _context19, _context20;
  var checkedKeys = {};
  var halfCheckedKeys = {}; // Record the key has some child checked (include child half checked)
  var checkStatus = status || {},
    oriCheckedKeys = checkStatus.checkedKeys || [],
    oriHalfCheckedKeys = checkStatus.halfCheckedKeys || [];
  _forEachInstanceProperty(oriCheckedKeys).call(oriCheckedKeys, function (key) {
    checkedKeys[key] = true;
  });
  _forEachInstanceProperty(oriHalfCheckedKeys).call(oriHalfCheckedKeys, function (key) {
    halfCheckedKeys[key] = true;
  });
  // Conduct up
  function conductUp(key) {
    var _context14, _context15;
    if (checkedKeys[key] === isCheck) return;
    var entity = keyEntities[key];
    if (!entity) return;
    var children = entity.children,
      parent = entity.parent,
      node = entity.node;
    if (isCheckDisabled(node)) return;
    // Check child node checked status
    var everyChildChecked = true;
    _forEachInstanceProperty(_context14 = _filterInstanceProperty(_context15 = children || []).call(_context15, function (child) {
      return !isCheckDisabled(child.node);
    })).call(_context14, function (child) {
      var childKey = child.key;
      if (!checkedKeys[childKey]) {
        everyChildChecked = false;
      }
    });
    // Update checked status
    if (everyChildChecked) {
      checkedKeys[key] = true;
      if (parent) {
        conductUp(parent.key);
      }
    }
  }
  function conduct(key) {
    var _context16, _context17;
    var entity = keyEntities[key];
    if (!entity) {
      warning(false, "'".concat(key, "' does not exist in the tree."));
      return;
    }
    var children = entity.children,
      parent = entity.parent,
      node = entity.node;
    checkedKeys[key] = isCheck;
    if (isCheckDisabled(node)) return;
    var everyChildChecked = true;
    // Conduct down
    _forEachInstanceProperty(_context16 = _filterInstanceProperty(_context17 = children || []).call(_context17, function (child) {
      return !isCheckDisabled(child.node);
    })).call(_context16, function (child) {
      var childKey = child.key;
      var entity = keyEntities[childKey];
      if (!entity) return;
      var children = entity.children,
        node = entity.node;
      if (isCheckDisabled(node)) return;
      if (!checkedKeys[childKey]) {
        everyChildChecked = false;
      }
    });
    // Update checked status
    if (everyChildChecked) {
      checkedKeys[key] = true;
      // Conduct up
      if (parent) {
        conductUp(parent.key);
      }
    }
  }
  _forEachInstanceProperty(_context18 = keyList || []).call(_context18, function (key) {
    conduct(key);
  });
  var checkedKeyList = [];
  var halfCheckedKeyList = [];
  // Fill checked list
  _forEachInstanceProperty(_context19 = _Object$keys(checkedKeys)).call(_context19, function (key) {
    if (checkedKeys[key]) {
      checkedKeyList.push(key);
    }
  });
  // Fill half checked list
  _forEachInstanceProperty(_context20 = _Object$keys(halfCheckedKeys)).call(_context20, function (key) {
    if (!checkedKeys[key] && halfCheckedKeys[key]) {
      halfCheckedKeyList.push(key);
    }
  });
  return {
    checkedKeys: checkedKeyList,
    halfCheckedKeys: halfCheckedKeyList
  };
}
/**
 * If user use `autoExpandParent` we should get the list of parent node
 * @param keyList
 * @param keyEntities
 */
export function conductExpandParent(keyList, keyEntities) {
  var _context21;
  var expandedKeys = {};
  function conductUp(key) {
    if (expandedKeys[key]) return;
    var entity = keyEntities[key];
    if (!entity) return;
    expandedKeys[key] = true;
    var parent = entity.parent,
      node = entity.node;
    if (isCheckDisabled(node)) return;
    if (parent) {
      conductUp(parent.key);
    }
  }
  _forEachInstanceProperty(_context21 = keyList || []).call(_context21, function (key) {
    conductUp(key);
  });
  return _Object$keys(expandedKeys);
}
/**
 * Returns only the data- and aria- key/value pairs
 * @param {object} props
 */
export function getDataAndAria(props) {
  var _context22;
  return _reduceInstanceProperty(_context22 = _Object$keys(props)).call(_context22, function (prev, key) {
    if (key.substr(0, 5) === 'data-' || key.substr(0, 5) === 'aria-') {
      prev[key] = props[key];
    }
    return prev;
  }, {});
}