import _typeof from "@babel/runtime-corejs3/helpers/typeof";
import _indexOfInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/index-of";
import _Object$getOwnPropertySymbols from "@babel/runtime-corejs3/core-js-stable/object/get-own-property-symbols";
import _forEachInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/for-each";
import _Array$isArray from "@babel/runtime-corejs3/core-js-stable/array/is-array";
import _mapInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/map";
import _sliceInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/slice";
import _Object$assign from "@babel/runtime-corejs3/core-js-stable/object/assign";
import _sortInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/sort";
import _Object$keys from "@babel/runtime-corejs3/core-js-stable/object/keys";
import _concatInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/concat";
import _filterInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/filter";
var __rest = this && this.__rest || function (s, e) {
  var t = {};
  for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && _indexOfInstanceProperty(e).call(e, p) < 0) t[p] = s[p];
  if (s != null && typeof _Object$getOwnPropertySymbols === "function") for (var i = 0, p = _Object$getOwnPropertySymbols(s); i < p.length; i++) {
    if (_indexOfInstanceProperty(e).call(e, p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i])) t[p[i]] = s[p[i]];
  }
  return t;
};
import React from 'react';
import warning from 'warning';
import { convertDataToTree as rcConvertDataToTree, convertTreeToEntities as rcConvertTreeToEntities, conductCheck as rcConductCheck } from '../rcTree/util.js';
import SelectNode from './SelectNode.js';
import { SHOW_CHILD, SHOW_PARENT } from './strategies.js';
var warnDeprecatedLabel = false;
export function toNodeArray(children) {
  var _context;
  var ret = [];
  _forEachInstanceProperty(_context = React.Children).call(_context, children, function (c) {
    ret.push(c);
  });
  return ret;
}
// =================== MISC ====================
export function toTitle(title) {
  if (typeof title === 'string') {
    return title;
  }
  return null;
}
export function toArray(data) {
  if (!data) return [];
  return _Array$isArray(data) ? data : [data];
}
// Shallow copy of React 16.3 createRef api
export function createRef() {
  var func = function setRef(node) {
    func.current = node;
  };
  return func;
}
// =============== Legacy ===============
export var UNSELECTABLE_STYLE = {
  userSelect: 'none',
  WebkitUserSelect: 'none'
};
export var UNSELECTABLE_ATTRIBUTE = {
  unselectable: 'unselectable'
};
/**
 * Convert position list to hierarchy structure.
 * This is little hack since use '-' to split the position.
 */
export function flatToHierarchy(positionList) {
  var _context2, _context4;
  if (!positionList.length) {
    return [];
  }
  var entrances = {};
  // Prepare the position map
  var posMap = {};
  var parsedList = _mapInstanceProperty(_context2 = _sliceInstanceProperty(positionList).call(positionList)).call(_context2, function (entity) {
    var clone = _Object$assign(_Object$assign({}, entity), {
      fields: entity.pos.split('-')
    });
    delete clone.children;
    return clone;
  });
  _forEachInstanceProperty(parsedList).call(parsedList, function (entity) {
    posMap[entity.pos] = entity;
  });
  // 按照节点的位置排序
  _sortInstanceProperty(parsedList).call(parsedList, function (a, b) {
    var aLen = a.fields.length,
      bLen = b.fields.length,
      minLen = aLen <= bLen ? aLen : bLen;
    for (var i = 0; i < minLen; i++) {
      var aPos = a.fields[i],
        bPos = b.fields[i];
      if (aPos == bPos) {
        if (i == minLen - 1) {
          return -1;
        }
      } else {
        return aPos - bPos;
      }
    }
  });
  // Create the hierarchy
  _forEachInstanceProperty(parsedList).call(parsedList, function (entity) {
    var _context3;
    var parentPos = _sliceInstanceProperty(_context3 = entity.fields).call(_context3, 0, -1).join('-');
    var parentEntity = posMap[parentPos];
    if (!parentEntity) {
      entrances[entity.pos] = entity;
    } else {
      parentEntity.children = parentEntity.children || [];
      parentEntity.children.push(entity);
    }
    // Some time position list provide `key`, we don't need it
    delete entity.key;
    delete entity.fields;
  });
  return _mapInstanceProperty(_context4 = _Object$keys(entrances)).call(_context4, function (key) {
    return entrances[key];
  });
}
// =============== Accessibility ===============
var ariaId = 0;
export function resetAriaId() {
  ariaId = 0;
}
export function generateAriaId(prefix) {
  var _context5;
  ariaId += 1;
  return _concatInstanceProperty(_context5 = "".concat(prefix, "_")).call(_context5, ariaId);
}
export function isLabelInValue(props) {
  var treeCheckable = props.treeCheckable,
    treeCheckStrictly = props.treeCheckStrictly,
    labelInValue = props.labelInValue;
  if (treeCheckable && treeCheckStrictly) {
    return true;
  }
  return labelInValue || false;
}
// =================== Tree ====================
export function parseSimpleTreeData(treeData, _ref) {
  var id = _ref.id,
    pId = _ref.pId,
    rootPId = _ref.rootPId;
  var keyNodes = {};
  var rootNodeList = [];
  // Fill in the map
  var nodeList = _mapInstanceProperty(treeData).call(treeData, function (node) {
    var clone = _Object$assign({}, node);
    var key = clone[id];
    keyNodes[key] = clone;
    clone.key = clone.key || key;
    return clone;
  });
  // Connect tree
  _forEachInstanceProperty(nodeList).call(nodeList, function (node) {
    var parentKey = node[pId];
    var parent = keyNodes[parentKey];
    // Fill parent
    if (parent) {
      parent.children = parent.children || [];
      parent.children.push(node);
    }
    // Fill root tree node
    if (parentKey === rootPId || !parent && rootPId === null) {
      rootNodeList.push(node);
    }
  });
  return rootNodeList;
}
/**
 * Detect if position has relation.
 * e.g. 1-2 related with 1-2-3
 * e.g. 1-3-2 related with 1
 * e.g. 1-2 not related with 1-21
 */
export function isPosRelated(pos1, pos2) {
  var fields1 = pos1.split('-');
  var fields2 = pos2.split('-');
  var minLen = Math.min(fields1.length, fields2.length);
  for (var i = 0; i < minLen; i += 1) {
    if (fields1[i] !== fields2[i]) {
      return false;
    }
  }
  return true;
}
/**
 * This function is only used on treeNode check (none treeCheckStrictly but has searchInput).
 * We convert entity to { node, pos, children } format.
 * This is legacy bug but we still need to do with it.
 * @param entity
 */
export function cleanEntity(_ref2) {
  var node = _ref2.node,
    pos = _ref2.pos,
    children = _ref2.children;
  var instance = {
    node: node,
    pos: pos
  };
  if (children) {
    instance.children = _mapInstanceProperty(children).call(children, cleanEntity);
  }
  return instance;
}
/**
 * Get a filtered TreeNode list by provided treeNodes.
 * [Legacy] Since `Tree` use `key` as map but `key` will changed by React,
 * we have to convert `treeNodes > data > treeNodes` to keep the key.
 * Such performance hungry!
 */
export function getFilterTree(treeNodes, searchValue, filterFunc, valueEntities) {
  var _context8;
  if (!searchValue) {
    return null;
  }
  function mapFilteredNodeToData(node) {
    var _context6, _context7;
    if (!node) return null;
    var match = false;
    if (filterFunc(searchValue, node)) {
      match = true;
    }
    // Fixed checked status error in search result
    // const children = (React.Children.map(node.props.children, mapFilteredNodeToData) || []).filter(n => n);
    var children = _filterInstanceProperty(_context6 = _mapInstanceProperty(_context7 = toNodeArray(node.props.children)).call(_context7, mapFilteredNodeToData)).call(_context6, function (n) {
      return n;
    });
    if (children.length || match) {
      return /*#__PURE__*/React.createElement(SelectNode, _Object$assign({}, node.props, {
        key: valueEntities[node.props.value].key
      }), children);
    }
    return null;
  }
  return _filterInstanceProperty(_context8 = _mapInstanceProperty(treeNodes).call(treeNodes, mapFilteredNodeToData)).call(_context8, function (node) {
    return node;
  });
}
// =================== Value ===================
/**
 * Convert value to array format to make logic simplify.
 */
export function formatInternalValue(value, props) {
  var valueList = toArray(value);
  // Parse label in value
  if (isLabelInValue(props)) {
    return _mapInstanceProperty(valueList).call(valueList, function (val) {
      if (_typeof(val) !== 'object' || !val) {
        return {
          value: '',
          label: ''
        };
      }
      return val;
    });
  }
  return _mapInstanceProperty(valueList).call(valueList, function (val) {
    return {
      value: val
    };
  });
}
export function getLabel(wrappedValue, entity, treeNodeLabelProp) {
  if (wrappedValue.label) {
    return wrappedValue.label;
  }
  if (entity && entity.node.props) {
    return entity.node.props[treeNodeLabelProp];
  }
  // Since value without entity will be in missValueList.
  // This code will never reached, but we still need this in case.
  return wrappedValue.value;
}
/**
 * Convert internal state `valueList` to user needed value list.
 * This will return an array list. You need check if is not multiple when return.
 * `allCheckedNodes` is used for `treeCheckStrictly`
 */
export function formatSelectorValue(valueList, props, valueEntities) {
  var treeNodeLabelProp = props.treeNodeLabelProp,
    treeCheckable = props.treeCheckable,
    treeCheckStrictly = props.treeCheckStrictly,
    showCheckedStrategy = props.showCheckedStrategy;
  // Will hide some value if `showCheckedStrategy` is set
  if (treeCheckable && !treeCheckStrictly) {
    var values = {};
    _forEachInstanceProperty(valueList).call(valueList, function (wrappedValue) {
      values[wrappedValue.value] = wrappedValue;
    });
    var hierarchyList = flatToHierarchy(_mapInstanceProperty(valueList).call(valueList, function (_ref3) {
      var value = _ref3.value;
      return valueEntities[value];
    }));
    if (showCheckedStrategy === SHOW_PARENT) {
      // 返回除 label、value 外更多的信息
      // Only get the parent checked value
      return _mapInstanceProperty(hierarchyList).call(hierarchyList, function (_a) {
        var _b = _a.node.props,
          label = _b.label,
          value = _b.value,
          restProps = __rest(_b, ["label", "value"]);
        return _Object$assign({
          label: getLabel(values[value], valueEntities[value], treeNodeLabelProp),
          value: value
        }, restProps);
      });
    } else if (showCheckedStrategy === SHOW_CHILD) {
      // Only get the children checked value
      var targetValueList = [];
      // Find the leaf children
      var traverse = function traverse(_a) {
        var _b = _a.node.props,
          label = _b.label,
          value = _b.value,
          restProps = __rest(_b, ["label", "value"]),
          children = _a.children;
        if (!children || children.length === 0) {
          // 返回除 label、value 外更多的信息
          targetValueList.push(_Object$assign({
            label: getLabel(values[value], valueEntities[value], treeNodeLabelProp),
            value: value
          }, restProps));
          return;
        }
        _forEachInstanceProperty(children).call(children, function (entity) {
          traverse(entity);
        });
      };
      _forEachInstanceProperty(hierarchyList).call(hierarchyList, function (entity) {
        traverse(entity);
      });
      return targetValueList;
    }
  }
  // return valueList.map(wrappedValue => ({
  //   label: getLabel(wrappedValue, valueEntities[wrappedValue.value], treeNodeLabelProp),
  //   value: wrappedValue.value,
  // }));
  // 返回除 label、value 外更多的信息
  return _mapInstanceProperty(valueList).call(valueList, function (wrappedValue) {
    var label = wrappedValue.label,
      value = wrappedValue.value,
      restWrappedValue = __rest(wrappedValue, ["label", "value"]);
    return _Object$assign({
      label: getLabel(wrappedValue, valueEntities[wrappedValue.value], treeNodeLabelProp),
      value: wrappedValue.value
    }, restWrappedValue);
  });
}
/**
 * Use `rc-tree` convertDataToTree to convert treeData to TreeNodes.
 * This will change the label to title value
 */
function processProps(props) {
  var title = props.title,
    label = props.label,
    key = props.key,
    value = props.value;
  var cloneProps = _Object$assign({}, props);
  // Warning user not to use deprecated label prop.
  if (label && !title) {
    if (!warnDeprecatedLabel) {
      warning(false, '\'label\' in treeData is deprecated. Please use \'title\' instead.');
      warnDeprecatedLabel = true;
    }
    cloneProps.title = label;
  }
  if (!key) {
    cloneProps.key = value;
  }
  return cloneProps;
}
export function convertDataToTree(treeData) {
  return rcConvertDataToTree(treeData, {
    processProps: processProps
  });
}
/**
 * Use `rc-tree` convertTreeToEntities for entities calculation.
 * We have additional entities of `valueEntities`
 */
function initWrapper(wrapper) {
  return _Object$assign(_Object$assign({}, wrapper), {
    valueEntities: {}
  });
}
function processEntity(entity, wrapper) {
  var value = entity.node.props.value;
  entity.value = value;
  wrapper.valueEntities[value] = entity;
}
export function convertTreeToEntities(treeNodes) {
  return rcConvertTreeToEntities(treeNodes, {
    initWrapper: initWrapper,
    processEntity: processEntity
  });
}
export var conductCheck = rcConductCheck;