import _mapInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/map";
import _indexOfInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/index-of";
import _trimInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/trim";
import _parseInt from "@babel/runtime-corejs3/core-js-stable/parse-int";
import _Array$isArray from "@babel/runtime-corejs3/core-js-stable/array/is-array";
import _forEachInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/for-each";
import _concatInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/concat";
import DateUtils from './date';
import Locale from './locale';
var t = Locale.t;
var weeks = ['sun', 'mon', 'tue', 'wed', 'thu', 'fri', 'sat'];
var months = ['jan', 'feb', 'mar', 'apr', 'may', 'jun', 'jul', 'aug', 'sep', 'oct', 'nov', 'dec'];
DateUtils.i18n = {
  dayNamesShort: _mapInstanceProperty(weeks).call(weeks, function (week) {
    return t("fishd.datepicker.weeks.".concat(week));
  }),
  dayNames: _mapInstanceProperty(weeks).call(weeks, function (week) {
    return t("fishd.datepicker.weeks.".concat(week));
  }),
  monthNamesShort: _mapInstanceProperty(months).call(months, function (month) {
    return t("fishd.datepicker.months.".concat(month));
  }),
  monthNames: _mapInstanceProperty(months).call(months, function (month, index) {
    return t("fishd.datepicker.month".concat(index + 1));
  })
};
// 时间format
export var timeFormat = function timeFormat(format) {
  if (format && _indexOfInstanceProperty(format).call(format, 'ss') === -1) {
    return 'HH:mm';
  } else {
    return 'HH:mm:ss';
  }
};
// 日期format
export var dateFormat = function dateFormat(format) {
  var _context;
  if (format) return _trimInstanceProperty(_context = format.replace('HH:mm', '').replace(':ss', '')).call(_context);else return 'yyyy-MM-dd';
};
// 年份数组
export var YEARS_ARRAY = function YEARS_ARRAY() {
  var N = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 50;
  var result = [];
  var currentYearNum = new Date().getFullYear();
  var start = Math.max(currentYearNum - _parseInt(N / 2), 0);
  for (var i = 0; i < N; i++) {
    result.push(start + i);
  }
  return result;
};
// 判断两个日期是否相等
export var equalDate = function equalDate(dateA, dateB) {
  return dateA === dateB || new Date(dateA).getTime() === new Date(dateB).getTime();
};
// 判断两个日期数组是否相等
export var equalDateArr = function equalDateArr(arrA, arrB) {
  return arrA === arrB || _Array$isArray(arrA) && _Array$isArray(arrB) && arrA.length === 2 && arrB.length === 2 && equalDate(arrA[0], arrB[0]) && equalDate(arrA[1], arrB[1]);
};
// 判断两个日期的年\月是否相等
export var equalYearAndMonth = function equalYearAndMonth(dateA, dateB) {
  return dateA.getFullYear() === dateB.getFullYear() && dateA.getMonth() === dateB.getMonth();
};
// 判断两个日期差(天)
export var diffDate = function diffDate(dateA, dateB) {
  var time = dateB.getTime() - dateA.getTime();
  var days = _parseInt(time / (1000 * 60 * 60 * 24));
  return days;
};
// Date对象
export var toDate = function toDate(date) {
  return isValidValue(date) ? new Date(date) : null;
};
// 判断值的合法性：Date合法
export var isValidValue = function isValidValue(value) {
  if (value instanceof Date) return true;
  return false;
};
// 判断值的合法性：[Date,Date]合法
export var isValidValueArr = function isValidValueArr(value) {
  if (_Array$isArray(value) && value.length >= 2 && value[0] instanceof Date && value[1] instanceof Date) return true;
  return false;
};
// Date对象->string
export var formatDate = function formatDate(date, format) {
  date = toDate(date);
  if (!date) return '';
  return DateUtils.format(date, format || 'yyyy-MM-dd');
};
// string->Date对象
export var parseDate = function parseDate(string, format) {
  return DateUtils.parse(string, format || 'yyyy-MM-dd');
};
// 只改变时间，不改变日期
export var setTime = function setTime(oldDate, newDate) {
  var old = new Date(oldDate.getTime());
  var hour = newDate.getHours();
  var minute = newDate.getMinutes();
  var second = newDate.getSeconds();
  var milliSeconds = newDate.getMilliseconds();
  old.setHours(hour);
  old.setMinutes(minute);
  old.setSeconds(second);
  old.setMilliseconds(milliSeconds);
  return new Date(old.getTime());
};
export var getDayCountOfMonth = function getDayCountOfMonth(year, month) {
  if (month === 3 || month === 5 || month === 8 || month === 10) {
    return 30;
  }
  if (month === 1) {
    if (year % 4 === 0 && year % 100 !== 0 || year % 400 === 0) {
      return 29;
    } else {
      return 28;
    }
  }
  return 31;
};
export var getFirstDayOfMonth = function getFirstDayOfMonth(date) {
  var temp = new Date(date.getTime());
  temp.setDate(1);
  return temp.getDay();
};
export var DAY_DURATION = 86400000;
// return date corresponding to the first cell on datetable
export var getStartDateOfMonth = function getStartDateOfMonth(year, month) {
  var offsetWeek = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;
  var result = new Date(year, month, 1);
  var day = result.getDay();
  if (day === offsetWeek) {
    result.setTime(result.getTime() - DAY_DURATION * 7);
  } else {
    var offset = getOffsetToWeekOrigin(day, offsetWeek);
    result.setTime(result.getTime() - DAY_DURATION * offset);
  }
  return result;
};
/**
 *
 * @export
 * @param {any} day , first day of current month, 0 - 6
 * @param {number} [offsetWeek=0, 0-6, 0 sunday, 6 saturday]
 * @returns
 */
export function getOffsetToWeekOrigin(day) {
  var offsetWeek = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
  var offset = day >= offsetWeek ? day - offsetWeek : 7 + day - offsetWeek;
  offset = offset === 0 ? 7 : offset; // if the two days collide, we force 7 days padding
  return offset;
}
export var getWeekNumber = function getWeekNumber(src) {
  var date = new Date(src.getTime());
  date.setHours(0, 0, 0, 0);
  // Thursday in current week decides the year.
  date.setDate(date.getDate() + 3 - (date.getDay() + 6) % 7);
  // January 4 is always in week 1.
  var week1 = new Date(date.getFullYear(), 0, 4);
  // Adjust to Thursday in week 1 and count number of weeks from date to week 1.
  return 1 + Math.round(((date.getTime() - week1.getTime()) / 86400000 - 3 + (week1.getDay() + 6) % 7) / 7);
};
// http://stackoverflow.com/questions/16590500/javascript-calculate-date-from-week-number
export function getDateOfISOWeek(w, y) {
  var simple = new Date(y, 0, 1 + (w - 1) * 7);
  var dow = simple.getDay();
  var ISOweekStart = simple;
  if (dow <= 4) ISOweekStart.setDate(simple.getDate() - simple.getDay() + 1);else ISOweekStart.setDate(simple.getDate() + 8 - simple.getDay());
  return ISOweekStart;
}
export var prevYear = function prevYear(date) {
  var d = toDate(date);
  d.setFullYear(date.getFullYear() - 1);
  return d;
};
export var nextYear = function nextYear(date) {
  var d = toDate(date);
  d.setFullYear(date.getFullYear() + 1);
  return d;
};
export var prevMonth = function prevMonth(src) {
  var clone = new Date(src.getTime());
  var year = clone.getFullYear();
  var month = clone.getMonth();
  var date = clone.getDate();
  var newYear = month === 0 ? year - 1 : year;
  var newMonth = month === 0 ? 11 : month - 1;
  var newMonthDayCount = getDayCountOfMonth(newYear, newMonth);
  if (newMonthDayCount < date) {
    clone.setDate(newMonthDayCount);
  }
  clone.setMonth(newMonth);
  clone.setFullYear(newYear);
  return clone;
};
export var nextMonth = function nextMonth(src) {
  var clone = new Date(src.getTime());
  var year = clone.getFullYear();
  var month = clone.getMonth();
  var date = clone.getDate();
  var newYear = month === 11 ? year + 1 : year;
  var newMonth = month === 11 ? 0 : month + 1;
  var newMonthDayCount = getDayCountOfMonth(newYear, newMonth);
  if (newMonthDayCount < date) {
    clone.setDate(newMonthDayCount);
  }
  clone.setMonth(newMonth);
  clone.setFullYear(newYear);
  return clone;
};
export var getRangeHours = function getRangeHours(ranges) {
  var _context2;
  var newArray = function newArray(start, end) {
    var result = [];
    for (var i = start; i <= end; i++) {
      result.push(i);
    }
    return result;
  };
  var hours = [];
  var disabledHours = [];
  _forEachInstanceProperty(_context2 = ranges || []).call(_context2, function (range) {
    var value = _mapInstanceProperty(range).call(range, function (date) {
      return date.getHours();
    });
    disabledHours = _concatInstanceProperty(disabledHours).call(disabledHours, newArray(value[0], value[1]));
  });
  if (disabledHours.length) {
    for (var i = 0; i < 24; i++) {
      hours[i] = _indexOfInstanceProperty(disabledHours).call(disabledHours, i) === -1;
    }
  } else {
    for (var _i = 0; _i < 24; _i++) {
      hours[_i] = false;
    }
  }
  return hours;
};
export var limitRange = function limitRange(date, ranges) {
  var format = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'yyyy-MM-dd HH:mm:ss';
  if (!ranges || !ranges.length) return date;
  var len = ranges.length;
  date = DateUtils.parse(DateUtils.format(date, format), format);
  for (var i = 0; i < len; i++) {
    var range = ranges[i];
    if (date >= range[0] && date <= range[1]) {
      return date;
    }
  }
  var maxDate = ranges[0][0];
  var minDate = ranges[0][0];
  _forEachInstanceProperty(ranges).call(ranges, function (range) {
    minDate = new Date(Math.min(range[0], minDate));
    maxDate = new Date(Math.max(range[1], maxDate));
  });
  return date < minDate ? minDate : maxDate;
};
// 判断日期是否在范围以内
export var isLimitRange = function isLimitRange(date, ranges) {
  var format = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'yyyy-MM-dd HH:mm:ss';
  if (!ranges || !ranges.length) return true;
  var len = ranges.length;
  date = DateUtils.parse(DateUtils.format(date, format), format);
  for (var i = 0; i < len; i++) {
    var range = ranges[i];
    if (date >= range[0] && date <= range[1]) {
      return true;
    }
  }
  return false;
};
export function hasClass(target, classname) {
  return target.classList.contains(classname);
}
export var SELECTION_MODES = {
  YEAR: 'year',
  MONTH: 'month',
  WEEK: 'week',
  DAY: 'day',
  RANGE: 'range'
};
export function deconstructDate(date) {
  return {
    year: date.getFullYear(),
    month: date.getMonth(),
    week: getWeekNumber(date)
  };
}