import _indexOfInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/index-of";
import _trimInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/trim";
import EventObject from './EventObject';
/**
 * 是否能使用 DOM 方法
 * @type {Boolean}
 */
export var hasDOM = typeof window !== 'undefined' && !!window.document && !!document.createElement;
/**
 * 获取同级的相邻节点
 * @param {object} elem - 节点
 * @returns {Array} 相邻节点数组
 */
export var getSiblings = function getSiblings(elem) {
  var siblings = [];
  var sibling = elem.parentNode.firstChild;
  for (; sibling; sibling = sibling.nextSibling) if (sibling.nodeType === 1 && sibling !== elem) siblings.push(sibling);
  return siblings;
};
// Check if node is equal to root or in the subtree of root
export var contains = function contains(root, n) {
  var node = n;
  while (node) {
    if (node === root) {
      return true;
    }
    node = node.parentNode;
  }
  return false;
};
export var addEventListener = function addEventListener(target, eventType, callback) {
  function wrapCallback(e) {
    var ne = new EventObject(e);
    callback.call(target, ne);
  }
  if (target.addEventListener) {
    target.addEventListener(eventType, wrapCallback, false);
    return {
      remove: function remove() {
        target.removeEventListener(eventType, wrapCallback, false);
      }
    };
  } else if (target.attachEvent) {
    target.attachEvent("on".concat(eventType), wrapCallback);
    return {
      remove: function remove() {
        target.detachEvent("on".concat(eventType), wrapCallback);
      }
    };
  }
};
/**
 * 节点是否包含指定 className
 * @param  {Element}  node
 * @param  {String}  className
 * @return {Boolean}
 *
 * @example
 * dom.hasClass(document.body, 'foo');
 */
export function hasClass(node, className) {
  /* istanbul ignore if */
  if (!hasDOM || !node) {
    return false;
  }
  if (node.classList) {
    return node.classList.contains(className);
  } else {
    var _context;
    return _indexOfInstanceProperty(_context = node.className).call(_context, className) > -1;
  }
}
/**
 * 添加 className
 * @param {Element} node
 * @param {String} className
 *
 * @example
 * dom.addClass(document.body, 'foo');
 */
export function addClass(node, className, _force) {
  /* istanbul ignore if */
  if (!hasDOM || !node) {
    return;
  }
  if (node.classList) {
    node.classList.add(className);
  } else if (_force === true || !hasClass(node, className)) {
    node.className += " ".concat(className);
  }
}
/**
 * 移除 className
 * @param  {Element} node
 * @param  {String} className
 *
 * @example
 * dom.removeClass(document.body, 'foo');
 */
export function removeClass(node, className, _force) {
  /* istanbul ignore if */
  if (!hasDOM || !node) {
    return;
  }
  if (node.classList) {
    node.classList.remove(className);
  } else if (_force === true || hasClass(node, className)) {
    var _context2;
    node.className = _trimInstanceProperty(_context2 = node.className.replace(className, '').replace(/\s+/g, ' ')).call(_context2);
  }
}