import _typeof from "@babel/runtime-corejs3/helpers/typeof";
import _Object$keys from "@babel/runtime-corejs3/core-js-stable/object/keys";
import _bindInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/bind";
import _sliceInstanceProperty from "@babel/runtime-corejs3/core-js-stable/instance/slice";
import _Date$now from "@babel/runtime-corejs3/core-js-stable/date/now";
/**
 * 以浅拷贝的方式判断两个对象是否相等
 * @param objA
 * @param objB
 * @returns {bool}
 */
export var shallowEqual = function shallowEqual(objA, objB) {
  var _context;
  if (objA === objB) {
    return true;
  }
  if (_typeof(objA) !== 'object' || objA === null || _typeof(objB) !== 'object' || objB === null) {
    return false;
  }
  var keysA = _Object$keys(objA);
  var keysB = _Object$keys(objB);
  if (keysA.length !== keysB.length) {
    return false;
  }
  // Test for A's keys different from B.
  var bHasOwnProperty = _bindInstanceProperty(_context = Object.prototype.hasOwnProperty).call(_context, objB);
  for (var i = 0; i < keysA.length; i++) {
    if (!bHasOwnProperty(keysA[i]) || objA[keysA[i]] !== objB[keysA[i]]) {
      return false;
    }
  }
  return true;
};
export var shallowEqualArrays = function shallowEqualArrays(arrA, arrB) {
  if (arrA === arrB) {
    return true;
  }
  var len = arrA.length;
  if (arrB.length !== len) {
    return false;
  }
  for (var i = 0; i < len; i++) {
    if (arrA[i] !== arrB[i]) {
      return false;
    }
  }
  return true;
};
/**
 * 将HTML代码片段转化为编码后的字符串
 * @param htmlFragment
 * @returns {string}
 */
export var decodeHTML = function decodeHTML(htmlFragment) {
  var temp = document.createElement("div");
  temp.innerHTML = htmlFragment;
  var output = temp.innerText || temp.textContent;
  temp = null;
  return output;
};
export function createChainedFunction() {
  var args = _sliceInstanceProperty([]).call(arguments, 0);
  if (args.length === 1) {
    return args[0];
  }
  return function chainedFunction() {
    for (var i = 0; i < args.length; i++) {
      if (args[i] && args[i].apply) {
        args[i].apply(this, arguments);
      }
    }
  };
}
export function getScroll(target, top) {
  if (typeof window === 'undefined') {
    return 0;
  }
  var prop = top ? 'pageYOffset' : 'pageXOffset';
  var method = top ? 'scrollTop' : 'scrollLeft';
  var isWindow = target === window;
  var ret = isWindow ? target[prop] : target[method];
  // ie6,7,8 standard mode
  if (isWindow && typeof ret !== 'number') {
    ret = window.document.documentElement[method];
  }
  return ret;
}
var SPECIAL_CHARS_REGEXP = /([:\-_]+(.))/g;
var MOZ_HACK_REGEXP = /^moz([A-Z])/;
var camelCase = function camelCase(name) {
  return name.replace(SPECIAL_CHARS_REGEXP, function (_, separator, letter, offset) {
    return offset ? letter.toUpperCase() : letter;
  }).replace(MOZ_HACK_REGEXP, 'Moz$1');
};
/**
 * 获取元素的样式
 * @param  {[type]} element   [description] 元素标签
 * @param  {[type]} styleName [description] 样式名
 * @return {[type]}           [description]
 */
export function getStyle(element, styleName) {
  if (!element || !styleName) return null;
  styleName = camelCase(styleName);
  if (styleName === 'float') styleName = 'cssFloat';
  try {
    var computed = document.defaultView.getComputedStyle(element, '');
    return element.style[styleName] || computed ? computed[styleName] : null;
  } catch (e) {
    return element.style[styleName];
  }
}
/**
 * 生成全局唯一的id
 * @param  {String} [prefix=''] 前缀字符串
 * @return {String}
 *
 * @example
 * guid(); // j7jv509c
 * guid('prefix-'); // prefix-j7jv509d
 */
var timestamp = _Date$now();
export function guid(prefix) {
  prefix = prefix || '';
  return prefix + (timestamp++).toString(36);
}