"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.UNSELECTABLE_STYLE = exports.UNSELECTABLE_ATTRIBUTE = void 0;
exports.childrenToArray = childrenToArray;
exports.defaultFilterFn = defaultFilterFn;
exports.findFirstMenuItem = findFirstMenuItem;
exports.findIndexInValueBySingleValue = findIndexInValueBySingleValue;
exports.getLabelFromPropsValue = getLabelFromPropsValue;
exports.getMapKey = getMapKey;
exports.getPropValue = getPropValue;
exports.getSelectKeys = getSelectKeys;
exports.getValuePropValue = getValuePropValue;
exports.includesSeparators = includesSeparators;
exports.isCombobox = isCombobox;
exports.isMultiple = isMultiple;
exports.isMultipleOrTags = isMultipleOrTags;
exports.isMultipleOrTagsOrCombobox = isMultipleOrTagsOrCombobox;
exports.isSingleMode = isSingleMode;
exports.preventDefaultEvent = preventDefaultEvent;
exports.saveRef = saveRef;
exports.splitBySeparators = splitBySeparators;
exports.toArray = toArray;
exports.toTitle = toTitle;
exports.validateOptionValue = validateOptionValue;
var _typeof2 = _interopRequireDefault(require("@babel/runtime-corejs3/helpers/typeof"));
var _forEach = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/for-each"));
var _isArray = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/array/is-array"));
var _concat = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/concat"));
var _lastIndexOf = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/last-index-of"));
var _filter = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/filter"));
var _indexOf = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/index-of"));
var _react = _interopRequireDefault(require("react"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }
function toTitle(title) {
  if (typeof title === 'string') {
    return title;
  }
  return null;
}
function getValuePropValue(child) {
  if (!child) {
    return null;
  }
  var props = child.props;
  if ('value' in props) {
    return props.value;
  }
  if (child.key) {
    return child.key;
  }
  if (child.type && child.type.isSelectOptGroup && props.label) {
    return props.label;
  }
  throw new Error("Need at least a key or a value or a label (only for OptGroup) for ".concat(child));
}
function getPropValue(child, prop) {
  if (prop === 'value') {
    return getValuePropValue(child);
  }
  return child.props[prop];
}
function isMultiple(props) {
  return props.multiple;
}
function isCombobox(props) {
  return props.combobox;
}
function isMultipleOrTags(props) {
  return props.multiple || props.tags;
}
function isMultipleOrTagsOrCombobox(props) {
  return isMultipleOrTags(props) || isCombobox(props);
}
function isSingleMode(props) {
  return !isMultipleOrTagsOrCombobox(props);
}
function childrenToArray(children) {
  var _context;
  var ret = [];
  (0, _forEach["default"])(_context = _react["default"].Children).call(_context, children, function (c) {
    ret.push(c);
  });
  return ret;
}
function toArray(value) {
  var ret = value;
  if (value === undefined) {
    ret = [];
  } else if (!(0, _isArray["default"])(value)) {
    ret = [value];
  }
  return ret;
}
function getMapKey(value) {
  var _context2;
  return (0, _concat["default"])(_context2 = "".concat((0, _typeof2["default"])(value), "-")).call(_context2, value);
}
function preventDefaultEvent(e) {
  e.preventDefault();
}
function findIndexInValueBySingleValue(value, singleValue) {
  var index = -1;
  for (var i = 0; i < value.length; i++) {
    if (value[i] === singleValue) {
      index = i;
      break;
    }
  }
  return index;
}
function getLabelFromPropsValue(value, key) {
  var label;
  value = toArray(value);
  for (var i = 0; i < value.length; i++) {
    if (value[i].key === key) {
      label = value[i].label;
      break;
    }
  }
  return label;
}
function getSelectKeys(menuItems, value) {
  var _context3;
  if (value === null || value === undefined) {
    return [];
  }
  var selectedKeys = [];
  (0, _forEach["default"])(_context3 = _react["default"].Children).call(_context3, menuItems, function (item) {
    if (item.type.isMenuItemGroup) {
      selectedKeys = (0, _concat["default"])(selectedKeys).call(selectedKeys, getSelectKeys(item.props.children, value));
    } else {
      var itemValue = getValuePropValue(item);
      var itemKey = item.key;
      if (findIndexInValueBySingleValue(value, itemValue) !== -1 && itemKey) {
        selectedKeys.push(itemKey);
      }
    }
  });
  return selectedKeys;
}
var UNSELECTABLE_STYLE = {
  userSelect: 'none',
  WebkitUserSelect: 'none'
};
exports.UNSELECTABLE_STYLE = UNSELECTABLE_STYLE;
var UNSELECTABLE_ATTRIBUTE = {
  unselectable: 'on'
};
exports.UNSELECTABLE_ATTRIBUTE = UNSELECTABLE_ATTRIBUTE;
function findFirstMenuItem(children) {
  for (var i = 0; i < children.length; i++) {
    var child = children[i];
    if (child.type.isMenuItemGroup) {
      var found = findFirstMenuItem(child.props.children);
      if (found) {
        return found;
      }
    } else if (!child.props.disabled) {
      return child;
    }
  }
  return null;
}
function includesSeparators(string, separators) {
  for (var i = 0; i < separators.length; ++i) {
    if ((0, _lastIndexOf["default"])(string).call(string, separators[i]) > 0) {
      return true;
    }
  }
  return false;
}
function splitBySeparators(string, separators) {
  var _context4;
  var reg = new RegExp("[".concat(separators.join(), "]"));
  return (0, _filter["default"])(_context4 = string.split(reg)).call(_context4, function (token) {
    return token;
  });
}
function defaultFilterFn(input, child) {
  var _context5;
  if (child.props.disabled) {
    return false;
  }
  var value = toArray(getPropValue(child, this.props.optionFilterProp)).join('');
  return (0, _indexOf["default"])(_context5 = value.toLowerCase()).call(_context5, input.toLowerCase()) > -1;
}
function validateOptionValue(value, props) {
  if (isSingleMode(props) || isMultiple(props)) {
    return;
  }
  if (typeof value !== 'string') {
    throw new Error("Invalid `value` of type `".concat((0, _typeof2["default"])(value), "` supplied to Option, ") + "expected `string` when `tags/combobox` is `true`.");
  }
}
function saveRef(instance, name) {
  return function (node) {
    instance[name] = node;
  };
}