"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.argumentContainer = argumentContainer;
exports.flattenArray = flattenArray;
exports.flattenFields = flattenFields;
exports.getErrorStrs = getErrorStrs;
exports.getParams = getParams;
exports.getValidateTriggers = getValidateTriggers;
exports.getValueFromEvent = getValueFromEvent;
exports.hasRules = hasRules;
exports.identity = identity;
exports.isEmptyObject = isEmptyObject;
exports.normalizeValidateRules = normalizeValidateRules;
exports.startsWith = startsWith;
exports.treeTraverse = treeTraverse;
var _typeof2 = _interopRequireDefault(require("@babel/runtime-corejs3/helpers/typeof"));
var _concat = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/concat"));
var _isArray = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/array/is-array"));
var _forEach = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/for-each"));
var _keys = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/object/keys"));
var _map = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/map"));
var _assign = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/object/assign"));
var _reduce = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/reduce"));
var _filter = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/filter"));
var _some = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/some"));
var _lastIndexOf = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/last-index-of"));
var _hoistNonReactStatics = _interopRequireDefault(require("hoist-non-react-statics"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }
/* eslint-disable no-console */

function getDisplayName(WrappedComponent) {
  return WrappedComponent.displayName || WrappedComponent.name || 'WrappedComponent';
}
function argumentContainer(Container, WrappedComponent) {
  /* eslint no-param-reassign:0 */
  Container.displayName = "Form(".concat(getDisplayName(WrappedComponent), ")");
  Container.WrappedComponent = WrappedComponent;
  return (0, _hoistNonReactStatics["default"])(Container, WrappedComponent);
}
function identity(obj) {
  return obj;
}
function flattenArray(arr) {
  return (0, _concat["default"])(Array.prototype).apply([], arr);
}
function treeTraverse() {
  var path = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';
  var tree = arguments.length > 1 ? arguments[1] : undefined;
  var isLeafNode = arguments.length > 2 ? arguments[2] : undefined;
  var errorMessage = arguments.length > 3 ? arguments[3] : undefined;
  var callback = arguments.length > 4 ? arguments[4] : undefined;
  if (isLeafNode(path, tree)) {
    callback(path, tree);
  } else if (tree === undefined || tree === null) {
    return;
  } else if ((0, _isArray["default"])(tree)) {
    (0, _forEach["default"])(tree).call(tree, function (subTree, index) {
      var _context;
      return treeTraverse((0, _concat["default"])(_context = "".concat(path, "[")).call(_context, index, "]"), subTree, isLeafNode, errorMessage, callback);
    });
  } else {
    var _context2;
    // It's object and not a leaf node
    if ((0, _typeof2["default"])(tree) !== 'object') {
      console.error(errorMessage);
      return;
    }
    (0, _forEach["default"])(_context2 = (0, _keys["default"])(tree)).call(_context2, function (subTreeKey) {
      var _context3, _context4;
      var subTree = tree[subTreeKey];
      treeTraverse((0, _concat["default"])(_context3 = (0, _concat["default"])(_context4 = "".concat(path)).call(_context4, path ? '.' : '')).call(_context3, subTreeKey), subTree, isLeafNode, errorMessage, callback);
    });
  }
}
function flattenFields(maybeNestedFields, isLeafNode, errorMessage) {
  var fields = {};
  treeTraverse(undefined, maybeNestedFields, isLeafNode, errorMessage, function (path, node) {
    fields[path] = node;
  });
  return fields;
}
function normalizeValidateRules(validate, rules, validateTrigger) {
  var validateRules = (0, _map["default"])(validate).call(validate, function (item) {
    var newItem = (0, _assign["default"])((0, _assign["default"])({}, item), {
      trigger: item.trigger || []
    });
    if (typeof newItem.trigger === 'string') {
      newItem.trigger = [newItem.trigger];
    }
    return newItem;
  });
  if (rules) {
    var _context5;
    validateRules.push({
      trigger: validateTrigger ? (0, _concat["default"])(_context5 = []).call(_context5, validateTrigger) : [],
      rules: rules
    });
  }
  return validateRules;
}
function getValidateTriggers(validateRules) {
  var _context6, _context7;
  return (0, _reduce["default"])(_context6 = (0, _map["default"])(_context7 = (0, _filter["default"])(validateRules).call(validateRules, function (item) {
    return !!item.rules && item.rules.length;
  })).call(_context7, function (item) {
    return item.trigger;
  })).call(_context6, function (pre, curr) {
    return (0, _concat["default"])(pre).call(pre, curr);
  }, []);
}
function getValueFromEvent(e) {
  // To support custom element
  if (!e || !e.target) {
    return e;
  }
  var target = e.target;
  return target.type === 'checkbox' ? target.checked : target.value;
}
function getErrorStrs(errors) {
  if (errors) {
    return (0, _map["default"])(errors).call(errors, function (e) {
      if (e && e.message) {
        return e.message;
      }
      return e;
    });
  }
  return errors;
}
function getParams(ns, opt, cb) {
  var names = ns;
  var options = opt;
  var callback = cb;
  if (cb === undefined) {
    if (typeof names === 'function') {
      callback = names;
      options = {};
      names = undefined;
    } else if ((0, _isArray["default"])(names)) {
      if (typeof options === 'function') {
        callback = options;
        options = {};
      } else {
        options = options || {};
      }
    } else {
      callback = options;
      options = names || {};
      names = undefined;
    }
  }
  return {
    names: names,
    options: options,
    callback: callback
  };
}
function isEmptyObject(obj) {
  return (0, _keys["default"])(obj).length === 0;
}
function hasRules(validate) {
  if (validate) {
    return (0, _some["default"])(validate).call(validate, function (item) {
      return item.rules && item.rules.length;
    });
  }
  return false;
}
function startsWith(str, prefix) {
  return (0, _lastIndexOf["default"])(str).call(str, prefix, 0) === 0;
}