"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;
var _construct = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/reflect/construct"));
var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime-corejs3/helpers/classCallCheck"));
var _createClass2 = _interopRequireDefault(require("@babel/runtime-corejs3/helpers/createClass"));
var _inherits2 = _interopRequireDefault(require("@babel/runtime-corejs3/helpers/inherits"));
var _possibleConstructorReturn2 = _interopRequireDefault(require("@babel/runtime-corejs3/helpers/possibleConstructorReturn"));
var _getPrototypeOf2 = _interopRequireDefault(require("@babel/runtime-corejs3/helpers/getPrototypeOf"));
var _constants = require("../common/constants");
var _utils = require("../common/utils");
var _element = _interopRequireDefault(require("./element"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }
function _createSuper(Derived) {
  var hasNativeReflectConstruct = _isNativeReflectConstruct();
  return function _createSuperInternal() {
    var Super = (0, _getPrototypeOf2["default"])(Derived),
      result;
    if (hasNativeReflectConstruct) {
      var NewTarget = (0, _getPrototypeOf2["default"])(this).constructor;
      result = (0, _construct["default"])(Super, arguments, NewTarget);
    } else {
      result = Super.apply(this, arguments);
    }
    return (0, _possibleConstructorReturn2["default"])(this, result);
  };
}
function _isNativeReflectConstruct() {
  if (typeof Reflect === "undefined" || !_construct["default"]) return false;
  if (_construct["default"].sham) return false;
  if (typeof Proxy === "function") return true;
  try {
    Boolean.prototype.valueOf.call((0, _construct["default"])(Boolean, [], function () {}));
    return true;
  } catch (e) {
    return false;
  }
}
/**
 * Stage behind the highlighted element to give it a little
 * highlight from rest of the page
 */
var Stage = /*#__PURE__*/function (_Element) {
  (0, _inherits2["default"])(Stage, _Element);
  var _super = _createSuper(Stage);
  /**
   * @param {Object} options
   * @param {Window} window
   * @param {Document} document
   */
  function Stage(options, window, document) {
    var _this;
    (0, _classCallCheck2["default"])(this, Stage);
    _this = _super.call(this);
    _this.options = options;
    _this.window = window;
    _this.document = document;
    return _this;
  }
  /**
   * Prepares the DOM element if not already there
   * @private
   */
  (0, _createClass2["default"])(Stage, [{
    key: "attachNode",
    value: function attachNode() {
      var stage = this.document.getElementById(_constants.ID_STAGE);
      if (!stage) {
        stage = (0, _utils.createNodeFromString)((0, _constants.STAGE_HTML)());
        document.body.appendChild(stage);
      }
      this.node = stage;
      this.currentIndexNode = stage.querySelector(".".concat(_constants.CLASS_STAGE_INDEX));
      if (!this.options.animate) {
        this.node.classList.add(_constants.CLASS_STAGE_NO_ANIMATION);
      } else {
        this.node.classList.remove(_constants.CLASS_STAGE_NO_ANIMATION);
      }
    }
    /**
     * Simply hides the stage
     * @public
     */
  }, {
    key: "hide",
    value: function hide() {
      if (!this.node || !this.node.parentElement) {
        return;
      }
      this.node.parentElement.removeChild(this.node);
    }
    /**
     * Makes it visible and sets the default properties
     * @private
     */
  }, {
    key: "setInitialStyle",
    value: function setInitialStyle() {
      this.node.style.display = 'block';
      this.node.style.left = '0';
      this.node.style.top = '0';
      this.node.style.bottom = '';
      this.node.style.right = '';
    }
    /**
     * Shows the stage at provided position
     * @param {Position} position
     * @public
     */
  }, {
    key: "show",
    value: function show(position) {
      this.attachNode();
      this.setInitialStyle();
      // 展示或隐藏步骤计数器
      if (this.options.totalCount <= 1 || !this.options.counter) {
        this.currentIndexNode.style.display = 'none';
      } else {
        this.currentIndexNode.innerHTML = this.options.currentIndex + 1;
        this.currentIndexNode.style.display = 'block';
      }
      // 设置步骤计数器的位置
      switch (this.options.counterPosition) {
        case 'leftBottom':
          this.currentIndexNode.className = "".concat(_constants.CLASS_STAGE_INDEX, " left-bottom");
          break;
        case 'rightTop':
          this.currentIndexNode.className = "".concat(_constants.CLASS_STAGE_INDEX, " right-top");
          break;
        case 'rightBottom':
          this.currentIndexNode.className = "".concat(_constants.CLASS_STAGE_INDEX, " right-bottom");
          break;
        case 'leftTop':
        default:
          this.currentIndexNode.className = "".concat(_constants.CLASS_STAGE_INDEX);
          break;
      }
      // Make it two times the padding because, half will be given on left and half on right
      var requiredPadding = this.options.padding * 2;
      var width = position.right - position.left + requiredPadding;
      var height = position.bottom - position.top + requiredPadding;
      // Show the stage
      this.node.style.display = 'block';
      this.node.style.position = 'absolute';
      this.node.style.width = "".concat(width, "px");
      this.node.style.height = "".concat(height, "px");
      this.node.style.top = "".concat(position.top - requiredPadding / 2, "px");
      this.node.style.left = "".concat(position.left - requiredPadding / 2, "px");
      this.node.style.backgroundColor = this.options.stageBackground;
    }
  }]);
  return Stage;
}(_element["default"]);
exports["default"] = Stage;