"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;
var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime-corejs3/helpers/classCallCheck"));
var _createClass2 = _interopRequireDefault(require("@babel/runtime-corejs3/helpers/createClass"));
var _bind = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/bind"));
var _assign = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/object/assign"));
var _trim = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/trim"));
var _isArray = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/array/is-array"));
var _forEach = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/for-each"));
var _sort = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/sort"));
var _indexOf = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/index-of"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }
var FileDrop = /*#__PURE__*/function () {
  function FileDrop(quill) {
    var _context, _context2;
    var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {
      customDropFile: undefined,
      attachmentIconMap: undefined
    };
    (0, _classCallCheck2["default"])(this, FileDrop);
    this.quill = quill;
    this.customDropFile = options.customDropFile || null;
    this.attachmentIconMap = options.attachmentIconMap || {};
    this.handleDrop = (0, _bind["default"])(_context = this.handleDrop).call(_context, this);
    this.handlePaste = (0, _bind["default"])(_context2 = this.handlePaste).call(_context2, this);
    this.quill.root.addEventListener('drop', this.handleDrop, true);
    this.quill.root.addEventListener('paste', this.handlePaste, true);
  }
  (0, _createClass2["default"])(FileDrop, [{
    key: "handleDrop",
    value: function handleDrop(evt) {
      evt.preventDefault();
      if (evt.dataTransfer && evt.dataTransfer.files && evt.dataTransfer.files.length) {
        if (document.caretRangeFromPoint) {
          var selection = document.getSelection();
          var range = document.caretRangeFromPoint(evt.clientX, evt.clientY);
          if (selection && range) {
            selection.setBaseAndExtent(range.startContainer, range.startOffset, range.startContainer, range.startOffset);
          }
        }
        if (this.customDropFile && typeof this.customDropFile == 'function') {
          var _context3;
          this.customDropFile(evt.dataTransfer.files, (0, _bind["default"])(_context3 = this.insert).call(_context3, this));
        }
      }
    }
  }, {
    key: "handlePaste",
    value: function handlePaste(evt) {
      if (evt.clipboardData && evt.clipboardData.items && evt.clipboardData.items.length) {
        var hasFile = false,
          len = evt.clipboardData.items.length;
        for (var i = 0; i < len; i++) {
          var item = evt.clipboardData.items[i];
          if (item.kind == 'file') {
            hasFile = true;
            break;
          }
        }
        // 粘贴文件时禁止浏览器执行默认动作，防止粘贴后插入两次文件。粘贴非文件时执行浏览器默认的动作。
        if (hasFile) {
          evt.preventDefault();
          if (this.customDropFile && typeof this.customDropFile == 'function') {
            var _context4;
            this.customDropFile(evt.clipboardData.items, (0, _bind["default"])(_context4 = this.insert).call(_context4, this));
          }
        }
      }
    }
  }, {
    key: "handleFileInsert",
    value: function handleFileInsert(fileInfo) {
      if (!(fileInfo.url || fileInfo.src)) return;
      var range = this.quill.getSelection() || {},
        index = range.index != undefined ? range.index : this.quill.getLength(),
        curFormat = this.quill.getFormat(range);
      if (fileInfo.type == 'image') {
        // 插入图片
        if (!fileInfo.src) {
          return;
        }
        // 继承列表的样式
        var listFormat = {
          list: undefined
        };
        if (curFormat && curFormat.list) {
          listFormat.list = curFormat.list;
        }
        var delta = [{
          insert: {
            image: fileInfo
          },
          attributes: (0, _assign["default"])({}, listFormat)
        }, {
          insert: ' '
        } // 在图片后添加一个空格，避免图片与附件相邻时，再在图片后拖入附件，图片异常添加附件的样式
        ];

        if (index > 0) {
          delta.unshift({
            retain: index
          });
        }
        this.quill.updateContents(delta);
        this.quill.setSelection(index + 2);
      } else if (fileInfo.type == 'video') {
        // 插入视频
        if (!fileInfo.src) {
          return;
        }
        this.quill.insertEmbed(index, 'myVideo', fileInfo);
        this.quill.setSelection(index + 2);
      } else {
        var _context5;
        // 插入附件
        if (!fileInfo.url || !fileInfo.name) {
          return;
        }
        var attrs = {
          url: fileInfo.url && (0, _trim["default"])(_context5 = fileInfo.url).call(_context5),
          name: fileInfo.name,
          iconUrl: this.attachmentIconMap[fileInfo.type] || this.attachmentIconMap['default'] || '//res.qiyukf.net/operation/2edfafe507a11ad70724973bb505addd'
        };
        this.quill.insertEmbed(index, 'attach', attrs);
        this.quill.setSelection(index, 'silent');
      }
    }
  }, {
    key: "insert",
    value: function insert(fileList) {
      var _this = this;
      if ((0, _isArray["default"])(fileList)) {
        var _context6;
        (0, _forEach["default"])(_context6 = (0, _sort["default"])(fileList).call(fileList, function (a, b) {
          // 单次插入多个不同类型的文件时，按”视频 -> 图片 -> 其他文件“的顺序排列
          var order = ['other', 'image', 'video'];
          return (0, _indexOf["default"])(order).call(order, b.type) - (0, _indexOf["default"])(order).call(order, a.type);
        })).call(_context6, function (file) {
          file && _this.handleFileInsert(file);
        });
      } else {
        fileList && this.handleFileInsert(fileList);
      }
    }
  }]);
  return FileDrop;
}();
exports["default"] = FileDrop;