"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.UNSELECTABLE_STYLE = exports.UNSELECTABLE_ATTRIBUTE = void 0;
exports.cleanEntity = cleanEntity;
exports.conductCheck = void 0;
exports.convertDataToTree = convertDataToTree;
exports.convertTreeToEntities = convertTreeToEntities;
exports.createRef = createRef;
exports.flatToHierarchy = flatToHierarchy;
exports.formatInternalValue = formatInternalValue;
exports.formatSelectorValue = formatSelectorValue;
exports.generateAriaId = generateAriaId;
exports.getFilterTree = getFilterTree;
exports.getLabel = getLabel;
exports.isLabelInValue = isLabelInValue;
exports.isPosRelated = isPosRelated;
exports.parseSimpleTreeData = parseSimpleTreeData;
exports.resetAriaId = resetAriaId;
exports.toArray = toArray;
exports.toNodeArray = toNodeArray;
exports.toTitle = toTitle;
var _typeof2 = _interopRequireDefault(require("@babel/runtime-corejs3/helpers/typeof"));
var _indexOf = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/index-of"));
var _getOwnPropertySymbols = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/object/get-own-property-symbols"));
var _forEach = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/for-each"));
var _isArray = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/array/is-array"));
var _map = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/map"));
var _slice = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/slice"));
var _assign = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/object/assign"));
var _sort = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/sort"));
var _keys = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/object/keys"));
var _concat = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/concat"));
var _filter = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/filter"));
var _react = _interopRequireDefault(require("react"));
var _warning = _interopRequireDefault(require("warning"));
var _util = require("../rcTree/util.js");
var _SelectNode = _interopRequireDefault(require("./SelectNode.js"));
var _strategies = require("./strategies.js");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }
var __rest = void 0 && (void 0).__rest || function (s, e) {
  var t = {};
  for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && (0, _indexOf["default"])(e).call(e, p) < 0) t[p] = s[p];
  if (s != null && typeof _getOwnPropertySymbols["default"] === "function") for (var i = 0, p = (0, _getOwnPropertySymbols["default"])(s); i < p.length; i++) {
    if ((0, _indexOf["default"])(e).call(e, p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i])) t[p[i]] = s[p[i]];
  }
  return t;
};
var warnDeprecatedLabel = false;
function toNodeArray(children) {
  var _context;
  var ret = [];
  (0, _forEach["default"])(_context = _react["default"].Children).call(_context, children, function (c) {
    ret.push(c);
  });
  return ret;
}
// =================== MISC ====================
function toTitle(title) {
  if (typeof title === 'string') {
    return title;
  }
  return null;
}
function toArray(data) {
  if (!data) return [];
  return (0, _isArray["default"])(data) ? data : [data];
}
// Shallow copy of React 16.3 createRef api
function createRef() {
  var func = function setRef(node) {
    func.current = node;
  };
  return func;
}
// =============== Legacy ===============
var UNSELECTABLE_STYLE = {
  userSelect: 'none',
  WebkitUserSelect: 'none'
};
exports.UNSELECTABLE_STYLE = UNSELECTABLE_STYLE;
var UNSELECTABLE_ATTRIBUTE = {
  unselectable: 'unselectable'
};
/**
 * Convert position list to hierarchy structure.
 * This is little hack since use '-' to split the position.
 */
exports.UNSELECTABLE_ATTRIBUTE = UNSELECTABLE_ATTRIBUTE;
function flatToHierarchy(positionList) {
  var _context2, _context4;
  if (!positionList.length) {
    return [];
  }
  var entrances = {};
  // Prepare the position map
  var posMap = {};
  var parsedList = (0, _map["default"])(_context2 = (0, _slice["default"])(positionList).call(positionList)).call(_context2, function (entity) {
    var clone = (0, _assign["default"])((0, _assign["default"])({}, entity), {
      fields: entity.pos.split('-')
    });
    delete clone.children;
    return clone;
  });
  (0, _forEach["default"])(parsedList).call(parsedList, function (entity) {
    posMap[entity.pos] = entity;
  });
  // 按照节点的位置排序
  (0, _sort["default"])(parsedList).call(parsedList, function (a, b) {
    var aLen = a.fields.length,
      bLen = b.fields.length,
      minLen = aLen <= bLen ? aLen : bLen;
    for (var i = 0; i < minLen; i++) {
      var aPos = a.fields[i],
        bPos = b.fields[i];
      if (aPos == bPos) {
        if (i == minLen - 1) {
          return -1;
        }
      } else {
        return aPos - bPos;
      }
    }
  });
  // Create the hierarchy
  (0, _forEach["default"])(parsedList).call(parsedList, function (entity) {
    var _context3;
    var parentPos = (0, _slice["default"])(_context3 = entity.fields).call(_context3, 0, -1).join('-');
    var parentEntity = posMap[parentPos];
    if (!parentEntity) {
      entrances[entity.pos] = entity;
    } else {
      parentEntity.children = parentEntity.children || [];
      parentEntity.children.push(entity);
    }
    // Some time position list provide `key`, we don't need it
    delete entity.key;
    delete entity.fields;
  });
  return (0, _map["default"])(_context4 = (0, _keys["default"])(entrances)).call(_context4, function (key) {
    return entrances[key];
  });
}
// =============== Accessibility ===============
var ariaId = 0;
function resetAriaId() {
  ariaId = 0;
}
function generateAriaId(prefix) {
  var _context5;
  ariaId += 1;
  return (0, _concat["default"])(_context5 = "".concat(prefix, "_")).call(_context5, ariaId);
}
function isLabelInValue(props) {
  var treeCheckable = props.treeCheckable,
    treeCheckStrictly = props.treeCheckStrictly,
    labelInValue = props.labelInValue;
  if (treeCheckable && treeCheckStrictly) {
    return true;
  }
  return labelInValue || false;
}
// =================== Tree ====================
function parseSimpleTreeData(treeData, _ref) {
  var id = _ref.id,
    pId = _ref.pId,
    rootPId = _ref.rootPId;
  var keyNodes = {};
  var rootNodeList = [];
  // Fill in the map
  var nodeList = (0, _map["default"])(treeData).call(treeData, function (node) {
    var clone = (0, _assign["default"])({}, node);
    var key = clone[id];
    keyNodes[key] = clone;
    clone.key = clone.key || key;
    return clone;
  });
  // Connect tree
  (0, _forEach["default"])(nodeList).call(nodeList, function (node) {
    var parentKey = node[pId];
    var parent = keyNodes[parentKey];
    // Fill parent
    if (parent) {
      parent.children = parent.children || [];
      parent.children.push(node);
    }
    // Fill root tree node
    if (parentKey === rootPId || !parent && rootPId === null) {
      rootNodeList.push(node);
    }
  });
  return rootNodeList;
}
/**
 * Detect if position has relation.
 * e.g. 1-2 related with 1-2-3
 * e.g. 1-3-2 related with 1
 * e.g. 1-2 not related with 1-21
 */
function isPosRelated(pos1, pos2) {
  var fields1 = pos1.split('-');
  var fields2 = pos2.split('-');
  var minLen = Math.min(fields1.length, fields2.length);
  for (var i = 0; i < minLen; i += 1) {
    if (fields1[i] !== fields2[i]) {
      return false;
    }
  }
  return true;
}
/**
 * This function is only used on treeNode check (none treeCheckStrictly but has searchInput).
 * We convert entity to { node, pos, children } format.
 * This is legacy bug but we still need to do with it.
 * @param entity
 */
function cleanEntity(_ref2) {
  var node = _ref2.node,
    pos = _ref2.pos,
    children = _ref2.children;
  var instance = {
    node: node,
    pos: pos
  };
  if (children) {
    instance.children = (0, _map["default"])(children).call(children, cleanEntity);
  }
  return instance;
}
/**
 * Get a filtered TreeNode list by provided treeNodes.
 * [Legacy] Since `Tree` use `key` as map but `key` will changed by React,
 * we have to convert `treeNodes > data > treeNodes` to keep the key.
 * Such performance hungry!
 */
function getFilterTree(treeNodes, searchValue, filterFunc, valueEntities) {
  var _context8;
  if (!searchValue) {
    return null;
  }
  function mapFilteredNodeToData(node) {
    var _context6, _context7;
    if (!node) return null;
    var match = false;
    if (filterFunc(searchValue, node)) {
      match = true;
    }
    // Fixed checked status error in search result
    // const children = (React.Children.map(node.props.children, mapFilteredNodeToData) || []).filter(n => n);
    var children = (0, _filter["default"])(_context6 = (0, _map["default"])(_context7 = toNodeArray(node.props.children)).call(_context7, mapFilteredNodeToData)).call(_context6, function (n) {
      return n;
    });
    if (children.length || match) {
      return /*#__PURE__*/_react["default"].createElement(_SelectNode["default"], (0, _assign["default"])({}, node.props, {
        key: valueEntities[node.props.value].key
      }), children);
    }
    return null;
  }
  return (0, _filter["default"])(_context8 = (0, _map["default"])(treeNodes).call(treeNodes, mapFilteredNodeToData)).call(_context8, function (node) {
    return node;
  });
}
// =================== Value ===================
/**
 * Convert value to array format to make logic simplify.
 */
function formatInternalValue(value, props) {
  var valueList = toArray(value);
  // Parse label in value
  if (isLabelInValue(props)) {
    return (0, _map["default"])(valueList).call(valueList, function (val) {
      if ((0, _typeof2["default"])(val) !== 'object' || !val) {
        return {
          value: '',
          label: ''
        };
      }
      return val;
    });
  }
  return (0, _map["default"])(valueList).call(valueList, function (val) {
    return {
      value: val
    };
  });
}
function getLabel(wrappedValue, entity, treeNodeLabelProp) {
  if (wrappedValue.label) {
    return wrappedValue.label;
  }
  if (entity && entity.node.props) {
    return entity.node.props[treeNodeLabelProp];
  }
  // Since value without entity will be in missValueList.
  // This code will never reached, but we still need this in case.
  return wrappedValue.value;
}
/**
 * Convert internal state `valueList` to user needed value list.
 * This will return an array list. You need check if is not multiple when return.
 * `allCheckedNodes` is used for `treeCheckStrictly`
 */
function formatSelectorValue(valueList, props, valueEntities) {
  var treeNodeLabelProp = props.treeNodeLabelProp,
    treeCheckable = props.treeCheckable,
    treeCheckStrictly = props.treeCheckStrictly,
    showCheckedStrategy = props.showCheckedStrategy;
  // Will hide some value if `showCheckedStrategy` is set
  if (treeCheckable && !treeCheckStrictly) {
    var values = {};
    (0, _forEach["default"])(valueList).call(valueList, function (wrappedValue) {
      values[wrappedValue.value] = wrappedValue;
    });
    var hierarchyList = flatToHierarchy((0, _map["default"])(valueList).call(valueList, function (_ref3) {
      var value = _ref3.value;
      return valueEntities[value];
    }));
    if (showCheckedStrategy === _strategies.SHOW_PARENT) {
      // 返回除 label、value 外更多的信息
      // Only get the parent checked value
      return (0, _map["default"])(hierarchyList).call(hierarchyList, function (_a) {
        var _b = _a.node.props,
          label = _b.label,
          value = _b.value,
          restProps = __rest(_b, ["label", "value"]);
        return (0, _assign["default"])({
          label: getLabel(values[value], valueEntities[value], treeNodeLabelProp),
          value: value
        }, restProps);
      });
    } else if (showCheckedStrategy === _strategies.SHOW_CHILD) {
      // Only get the children checked value
      var targetValueList = [];
      // Find the leaf children
      var traverse = function traverse(_a) {
        var _b = _a.node.props,
          label = _b.label,
          value = _b.value,
          restProps = __rest(_b, ["label", "value"]),
          children = _a.children;
        if (!children || children.length === 0) {
          // 返回除 label、value 外更多的信息
          targetValueList.push((0, _assign["default"])({
            label: getLabel(values[value], valueEntities[value], treeNodeLabelProp),
            value: value
          }, restProps));
          return;
        }
        (0, _forEach["default"])(children).call(children, function (entity) {
          traverse(entity);
        });
      };
      (0, _forEach["default"])(hierarchyList).call(hierarchyList, function (entity) {
        traverse(entity);
      });
      return targetValueList;
    }
  }
  // return valueList.map(wrappedValue => ({
  //   label: getLabel(wrappedValue, valueEntities[wrappedValue.value], treeNodeLabelProp),
  //   value: wrappedValue.value,
  // }));
  // 返回除 label、value 外更多的信息
  return (0, _map["default"])(valueList).call(valueList, function (wrappedValue) {
    var label = wrappedValue.label,
      value = wrappedValue.value,
      restWrappedValue = __rest(wrappedValue, ["label", "value"]);
    return (0, _assign["default"])({
      label: getLabel(wrappedValue, valueEntities[wrappedValue.value], treeNodeLabelProp),
      value: wrappedValue.value
    }, restWrappedValue);
  });
}
/**
 * Use `rc-tree` convertDataToTree to convert treeData to TreeNodes.
 * This will change the label to title value
 */
function processProps(props) {
  var title = props.title,
    label = props.label,
    key = props.key,
    value = props.value;
  var cloneProps = (0, _assign["default"])({}, props);
  // Warning user not to use deprecated label prop.
  if (label && !title) {
    if (!warnDeprecatedLabel) {
      (0, _warning["default"])(false, '\'label\' in treeData is deprecated. Please use \'title\' instead.');
      warnDeprecatedLabel = true;
    }
    cloneProps.title = label;
  }
  if (!key) {
    cloneProps.key = value;
  }
  return cloneProps;
}
function convertDataToTree(treeData) {
  return (0, _util.convertDataToTree)(treeData, {
    processProps: processProps
  });
}
/**
 * Use `rc-tree` convertTreeToEntities for entities calculation.
 * We have additional entities of `valueEntities`
 */
function initWrapper(wrapper) {
  return (0, _assign["default"])((0, _assign["default"])({}, wrapper), {
    valueEntities: {}
  });
}
function processEntity(entity, wrapper) {
  var value = entity.node.props.value;
  entity.value = value;
  wrapper.valueEntities[value] = entity;
}
function convertTreeToEntities(treeNodes) {
  return (0, _util.convertTreeToEntities)(treeNodes, {
    initWrapper: initWrapper,
    processEntity: processEntity
  });
}
var conductCheck = _util.conductCheck;
exports.conductCheck = conductCheck;