import * as React from 'react';
import Option from './Option';
import OptGroup from './OptGroup';
import { LocaleProperties } from '../Locale';
interface SelectProps {
    allowClear?: boolean;
    children?: React.ReactNode | React.ReactNode[] | React.ReactChildren;
    className?: string;
    tagWidth?: string | number;
    defaultActiveFirstOption?: boolean;
    defaultValue?: string | number | any[] | object;
    disabled?: boolean;
    dropdownClassName?: string;
    dropdownMatchSelectWidth?: boolean;
    dropdownStyle?: React.CSSProperties;
    errorMessage?: string | React.ReactNode;
    extraOptions?: string | React.ReactNode;
    filterOption?: boolean | ((value: SelectValue, child?: React.ReactNode) => void);
    getPopupContainer?: (triggerNode: HTMLElement) => HTMLElement;
    labelClear?: boolean;
    labelInValue?: boolean;
    loading?: boolean;
    maxCount?: number;
    maxLabelClearPanelHeight?: string | number;
    maxScrollHeight?: string | number;
    mode?: 'multiple' | 'single';
    multipleSelectAllText?: string;
    notFoundContent?: string | React.ReactNode;
    onChange?: (value?: any) => void;
    onMouseEnter?: React.MouseEventHandler<HTMLDivElement>;
    onMouseLeave?: React.MouseEventHandler<HTMLDivElement>;
    onPopupScroll?: React.UIEventHandler<HTMLDivElement>;
    onSearch?: (value: any) => void;
    onSelect?: (value: any) => void;
    onDeselect?: (value: any) => void;
    onVisibleChange?: (value: any) => void;
    placeholder?: string;
    placement?: 'bottomLeft' | 'bottomCenter' | 'bottomRight' | 'topLeft' | 'topCenter' | 'topRight';
    prefixCls?: string;
    searchInputProps?: object;
    searchPlaceholder?: string;
    selectAllText?: string;
    showArrow?: boolean;
    showMultipleSelectAll?: boolean;
    showOptionCheckedIcon?: boolean;
    showSearch?: boolean;
    showSelectAll?: boolean;
    showSingleClear?: boolean;
    size?: 'default' | 'small' | 'large';
    style?: React.CSSProperties;
    value?: string | number | any[] | object;
    visible?: boolean;
    esc?: boolean;
    required?: boolean;
    filterInactiveOption?: boolean;
}
declare type SelectValue = {
    key: string;
    label: string;
    title: string;
};
interface SelectState {
    searchValue?: string;
    selectValue?: SelectValue[];
    selectValueForMultiplePanel?: SelectValue[];
    popupVisible?: boolean;
    activeKey?: undefined;
    dropdownWidth?: string | number;
    prevProps?: SelectProps;
}
declare class Select extends React.Component<SelectProps, SelectState> {
    static Option: typeof Option;
    static OptGroup: typeof OptGroup;
    private dropdownList;
    private trigger;
    private selection;
    private selectSearch;
    static propTypes: {
        allowClear: any;
        children: any;
        className: any;
        tagWidth: any;
        defaultActiveFirstOption: any;
        defaultValue: any;
        disabled: any;
        dropdownClassName: any;
        dropdownMatchSelectWidth: any;
        dropdownStyle: any;
        errorMessage: any;
        extraOptions: any;
        filterOption: any;
        getPopupContainer: any;
        labelClear: any;
        labelInValue: any;
        loading: any;
        maxCount: any;
        maxLabelClearPanelHeight: any;
        maxScrollHeight: any;
        mode: any;
        multipleSelectAllText: any;
        notFoundContent: any;
        onChange: any;
        onMouseEnter: any;
        onMouseLeave: any;
        onPopupScroll: any;
        onSearch: any;
        onSelect: any;
        onDeselect: any;
        onVisibleChange: any;
        placeholder: any;
        placement: any;
        prefixCls: any;
        searchInputProps: any;
        searchPlaceholder: any;
        selectAllText: any;
        showArrow: any;
        showMultipleSelectAll: any;
        showOptionCheckedIcon: any;
        showSearch: any;
        showSelectAll: any;
        showSingleClear: any;
        size: any;
        style: any;
        value: any;
        visible: any;
        esc: any;
        required: any;
        filterInactiveOption: any;
    };
    static contextType: React.Context<LocaleProperties>;
    static defaultProps: SelectProps;
    static getOptionFromChildren: (children: any, plainOptionList?: any[], filter?: any) => any[];
    static getValueFromProps: (value: any, labelInValue: any, children: any) => any;
    static getDerivedStateFromProps(nextProps: any, prevState: any): SelectState;
    constructor(props: any);
    componentDidMount(): void;
    componentDidUpdate(): void;
    setDropdownWidth: () => void;
    updateSearchValue: (e: any) => void;
    emptySearchValue: () => void;
    selectAllOption: () => void;
    emptySelectValue: () => void;
    changeVisibleState: (visible: any) => void;
    visibleChangeFromTrigger: (visible: any) => void;
    focusEvent: (event: any) => void;
    focus(): void;
    blur(): void;
    onOptionClick: (e: any, obj: any, clickInLabel: any) => void;
    getProcessedChildren: (children: any, dropdownCls: string) => any;
    getFilteredChildren: (children: any, ChildrenList?: any[]) => any[];
    handleCancelSelect: () => void;
    handleConfirmSelect: () => void;
    isSelectAll: (isMultiplePanel?: boolean) => boolean;
    handleKeyboardEvent: (e: any) => void;
    setActiveOptionIntoView: (activeKey: any) => void;
    resizeTrigger: () => void;
    getDropdownPanel(Locale: LocaleProperties['Select']): JSX.Element;
    getSelectionPanel(Locale: LocaleProperties['Select']): JSX.Element;
    render(): JSX.Element;
}
export default Select;
