"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ensureValueInRange = ensureValueInRange;
exports.ensureValuePrecision = ensureValuePrecision;
exports.getClosestPoint = getClosestPoint;
exports.getHandleCenterPosition = getHandleCenterPosition;
exports.getKeyboardValueMutator = getKeyboardValueMutator;
exports.getMousePosition = getMousePosition;
exports.getPrecision = getPrecision;
exports.getTouchPosition = getTouchPosition;
exports.isEventFromHandle = isEventFromHandle;
exports.isNotTouchEvent = isNotTouchEvent;
exports.isValueOutOfRange = isValueOutOfRange;
exports.pauseEvent = pauseEvent;
var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime-corejs3/helpers/toConsumableArray"));
var _some = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/some"));
var _keys = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/object/keys"));
var _map = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/map"));
var _parseFloat2 = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/parse-float"));
var _indexOf = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/index-of"));
var _reactDom = require("react-dom");
var _utils = require("../../utils");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }
function isEventFromHandle(e, handles) {
  var _context;
  return (0, _some["default"])(_context = (0, _keys["default"])(handles)).call(_context, function (key) {
    return e.target === (0, _reactDom.findDOMNode)(handles[key]);
  });
}
function isValueOutOfRange(value, _ref) {
  var min = _ref.min,
    max = _ref.max;
  return value < min || value > max;
}
function isNotTouchEvent(e) {
  return e.touches.length > 1 || e.type.toLowerCase() === 'touchend' && e.touches.length > 0;
}
function getClosestPoint(val, _ref2) {
  var _context2;
  var marks = _ref2.marks,
    step = _ref2.step,
    min = _ref2.min;
  var points = (0, _map["default"])(_context2 = (0, _keys["default"])(marks)).call(_context2, _parseFloat2["default"]);
  if (step !== null) {
    var closestStep = Math.round((val - min) / step) * step + min;
    points.push(closestStep);
  }
  var diffs = (0, _map["default"])(points).call(points, function (point) {
    return Math.abs(val - point);
  });
  return points[(0, _indexOf["default"])(diffs).call(diffs, Math.min.apply(Math, (0, _toConsumableArray2["default"])(diffs)))] || 0;
}
function getPrecision(step) {
  var stepString = step.toString();
  var precision = 0;
  if ((0, _indexOf["default"])(stepString).call(stepString, '.') >= 0) {
    precision = stepString.length - (0, _indexOf["default"])(stepString).call(stepString, '.') - 1;
  }
  return precision;
}
function getMousePosition(vertical, e) {
  return vertical ? e.clientY : e.pageX;
}
function getTouchPosition(vertical, e) {
  return vertical ? e.touches[0].clientY : e.touches[0].pageX;
}
function getHandleCenterPosition(vertical, handle) {
  var coords = handle.getBoundingClientRect();
  return vertical ? coords.top + coords.height * 0.5 : coords.left + coords.width * 0.5;
}
function ensureValueInRange(val, _ref3) {
  var max = _ref3.max,
    min = _ref3.min;
  if (val <= min) {
    return min;
  }
  if (val >= max) {
    return max;
  }
  return val;
}
function ensureValuePrecision(val, props) {
  var step = props.step;
  var closestPoint = getClosestPoint(val, props);
  return step === null ? closestPoint : (0, _parseFloat2["default"])(closestPoint.toFixed(getPrecision(step)));
}
function pauseEvent(e) {
  e.stopPropagation();
  e.preventDefault();
}
function getKeyboardValueMutator(e) {
  switch (e.keyCode) {
    case _utils.KeyCode.UP:
    case _utils.KeyCode.RIGHT:
      return function (value, props) {
        return value + props.step;
      };
    case _utils.KeyCode.DOWN:
    case _utils.KeyCode.LEFT:
      return function (value, props) {
        return value - props.step;
      };
    case _utils.KeyCode.END:
      return function (value, props) {
        return props.max;
      };
    case _utils.KeyCode.HOME:
      return function (value, props) {
        return props.min;
      };
    case _utils.KeyCode.PAGE_UP:
      return function (value, props) {
        return value + props.step * 2;
      };
    case _utils.KeyCode.PAGE_DOWN:
      return function (value, props) {
        return value - props.step * 2;
      };
    default:
      return undefined;
  }
}